/*
 * IGS memory mapped display driver
 */

#include "igs.h"
#include <linux/types.h>
#include <linux/fb.h>
#include <linux/kd.h>	// for KD_GRAPHICS
#include <asm/errno.h>	// for EINVAL
#include <asm/delay.h>	// for UDELAY
// in vt.c
extern int con_set_font(char * fontmap, int ch512);

// these command bytes prefix the command array entries in the igs.regs file
#define CRT_CMD 1
#define ATR_CMD 2
#define SEQ_CMD 3
#define GRF_CMD 4
#define MISC_CMD 5
#define EXT_CMD 6
#define PAL_CMD 7

static volatile char* pVidPort;
static int mode = 0;
static int nStartAddr = 0;
static int nCursorAddr = 0;

#define CMD(a,b,c) ((a<<16) | (b<<8) | (c)) 
#define GET_CRT(a) (pVidPort[0x3d4] = a, pVidPort[0x3d5])
#define SET_CRT(a,v) (pVidPort[0x3d4] = a, pVidPort[0x3d5] = (v))
#define GET_SEQ(a) (pVidPort[0x3c4] = a, pVidPort[0x3c5])

const int aTEXT[] = {
	CMD(MISC_CMD,0x00,0x67), // sync polarity(c0), clock select(0c), ram-enable(2), color(1)

	CMD(SEQ_CMD,0x00,0x01),	// sync & async reset: 0 to reset
	CMD(SEQ_CMD,0x01,0x00),	// clock mode, screen-off(20)
	CMD(SEQ_CMD,0x02,0x03), // write mask
	CMD(SEQ_CMD,0x03,0x00), // font select
	CMD(SEQ_CMD,0x04,0x03), // ext mem(2), chain4(8)
	CMD(SEQ_CMD,0x00,0x03),	// sync & async reset: 0 to reset

	CMD(CRT_CMD,0x11,0x0b),	// unlock CR0-7
	CMD(CRT_CMD,0,0x5f),	// h total
	CMD(CRT_CMD,1,0x4f),	// h disp end
	CMD(CRT_CMD,2,0x50),	// h blank start
	CMD(CRT_CMD,3,0x82),	// h blank end
	CMD(CRT_CMD,4,0x55),	// h retrace start
	CMD(CRT_CMD,5,0x81),	// h retrace end
	CMD(CRT_CMD,6,0xbf),	// v total
	CMD(CRT_CMD,7,0x1f),	// extra bits
	CMD(CRT_CMD,8,0x00),	// row scan
	CMD(CRT_CMD,9,0x4f),	// max scan line
	CMD(CRT_CMD,0x0A,0x0d),	// cursor start
	CMD(CRT_CMD,0x0B,0x0e),	// cursor end
	CMD(CRT_CMD,0x0C,0),	// start address (hi)
	CMD(CRT_CMD,0x0D,0),	// start address (lo)
	CMD(CRT_CMD,0x0E,0),	// cursor address
	CMD(CRT_CMD,0x0F,0),	// cursor address
	CMD(CRT_CMD,0x10,0x9c),	// v retrace start
	CMD(CRT_CMD,0x11,0x8e),	// v retrace end
	CMD(CRT_CMD,0x12,0x8f),	// v disp enable end
	CMD(CRT_CMD,0x13,0x28),	// offset
	CMD(CRT_CMD,0x14,0x1f),	// underline
	CMD(CRT_CMD,0x15,0x96),	// v blank start
	CMD(CRT_CMD,0x16,0xb9),	// v blank end
	CMD(CRT_CMD,0x17,0xa3),	// mode control
	CMD(CRT_CMD,0x18,0xff),	// line compare

	CMD(ATR_CMD,0x00,0x00),	// default palette
	CMD(ATR_CMD,0x01,0x01),
	CMD(ATR_CMD,0x02,0x02),
	CMD(ATR_CMD,0x03,0x03),
	CMD(ATR_CMD,0x04,0x04),
	CMD(ATR_CMD,0x05,0x05),
	CMD(ATR_CMD,0x06,0x06),
	CMD(ATR_CMD,0x07,0x07),
	CMD(ATR_CMD,0x08,0x08),
	CMD(ATR_CMD,0x09,0x09),
	CMD(ATR_CMD,0x0a,0x0a),
	CMD(ATR_CMD,0x0b,0x0b),
	CMD(ATR_CMD,0x0c,0x0c),
	CMD(ATR_CMD,0x0d,0x0d),
	CMD(ATR_CMD,0x0e,0x0e),
	CMD(ATR_CMD,0x0f,0x0f),
	CMD(ATR_CMD,0x10,0x0c),	// graphics(1), mono-attr(2), blink(8)
	CMD(ATR_CMD,0x11,0x00),	// overscan color
	CMD(ATR_CMD,0x12,0x0f),	// plane enable
	CMD(ATR_CMD,0x13,0x08),	// h pixel pan

	CMD(GRF_CMD,0x00,0x00),	// set/reset
	CMD(GRF_CMD,0x01,0x00),	// enable set/reset
	CMD(GRF_CMD,0x02,0x00),	// color compare
	CMD(GRF_CMD,0x03,0x00),	// data rotate
	CMD(GRF_CMD,0x04,0x00),	// read map select
	CMD(GRF_CMD,0x05,0x10),	// write mode(3), even/odd(10), srmode(20), mode256(40)
	CMD(GRF_CMD,0x06,0x02),	// graphics(1), chain(2), mmapaddress(c)
	CMD(GRF_CMD,0x07,0x00),	// color don't care
	CMD(GRF_CMD,0x08,0xff),	// bit mask

	CMD(EXT_CMD,0x10,0x10),	// ext start address
	CMD(EXT_CMD,0x11,0x00),	// crt overflow
	CMD(EXT_CMD,0x12,0x00),	// enable irq(1)
	CMD(EXT_CMD,0x13,0x00),
	CMD(EXT_CMD,0x14,0x51),	// vga mem fetches per row
	CMD(EXT_CMD,0x15,0x00),
//	CMD(EXT_CMD,0x30,0x01),	// zero wait(4), PCI burst(20), linear size (3)
	CMD(EXT_CMD,0x31,0x00),	// ext seg write
	CMD(EXT_CMD,0x32,0x00),	// ext seg read
	CMD(EXT_CMD,0x33,0x01),	// linear enable(1), segment on(10), 
	CMD(EXT_CMD,0x50,0x00),	// sprite control
	CMD(EXT_CMD,0x51,0x00),
	CMD(EXT_CMD,0x52,0x00),
	CMD(EXT_CMD,0x53,0x00),
	CMD(EXT_CMD,0x54,0x00),
	CMD(EXT_CMD,0x55,0x00),
	CMD(EXT_CMD,0x56,0x00),	// sprite cursor, DAC access (4)
	CMD(EXT_CMD,0x57,0x00),	// ext graphic mode(1)
	CMD(EXT_CMD,0x58,0x00),	// RAMDAC
	CMD(EXT_CMD,0x59,0x00),
	CMD(EXT_CMD,0x5a,0x00),
////	CMD(EXT_CMD,0x70,0x0b),	// slow mem(1), ras control(3e), 
////	CMD(EXT_CMD,0x71,0x10),	// was 0x63 == mem arch(3), sync mem(4), vram(8), mem-type(e0)=60
////	CMD(EXT_CMD,0x72,0x05),	// was 0x02 == mem size(3), bus wid(4), refresh(c0)
	CMD(EXT_CMD,0x73,0x30),	// fifo over(81), fifosize(8)
	CMD(EXT_CMD,0x74,0x1b),	// fifo size
	CMD(EXT_CMD,0x75,0x1e),	// fifo size
	CMD(EXT_CMD,0x76,0x00),	// eeprom 
	CMD(EXT_CMD,0x77,0x00),	// color depth(f): 256=1, 32k=6, 64k=2, 16M=4, 4G=3, 256l=9
	CMD(EXT_CMD,0x78,0x00),	
	CMD(EXT_CMD,0x79,0x00),
	CMD(EXT_CMD,0x7a,0xc8),	// cas control
	CMD(EXT_CMD,0x90,0x00),	// NOT graphics mode
	CMD(EXT_CMD,0xb0,0xd2),	// vclock numerator
	CMD(EXT_CMD,0xb1,0xce),	// vclock denom
////	CMD(EXT_CMD,0xb2,0xdb),	// memclock numerator
////	CMD(EXT_CMD,0xb3,0x54),	// memclock denom
	CMD(EXT_CMD,0xb9,0x80),	// toggle the clock pll params
	CMD(EXT_CMD,0xb9,0x00),

	CMD(PAL_CMD,0x00,0x04),	// 10=bypass the DAC
};

//---------------------------------------------------------------------------
//60 Hz
const int a640x480x1[] = {
	CMD(MISC_CMD,0x00,0xe3), // sync polarity(c0), clock select(0c), ram-enable(2), color(1)

	CMD(SEQ_CMD,0x00,0x01),	// sync & async reset: 0 to reset
	CMD(SEQ_CMD,0x01,0x01),	// clock mode, screen-off(20)
	CMD(SEQ_CMD,0x02,0x0f), // write mask
	CMD(SEQ_CMD,0x03,0x00), // font select
	CMD(SEQ_CMD,0x04,0x06), // ext mem(2), chain4(8)	!turn off chain4 for full screen access
	CMD(SEQ_CMD,0x00,0x03),	// sync & async reset: 0 to reset

	CMD(CRT_CMD,0x11,0x0b),	// unlock CR0-7
	CMD(CRT_CMD,0,0x5f),	// h total
	CMD(CRT_CMD,1,0x4f),	// h disp end
	CMD(CRT_CMD,2,0x50),	// h blank start
	CMD(CRT_CMD,3,0x82),	// h blank end
	CMD(CRT_CMD,4,0x54),	// h retrace start
	CMD(CRT_CMD,5,0x80),	// h retrace end
	CMD(CRT_CMD,6,0x0b),	// v total
	CMD(CRT_CMD,7,0x3e),	// extra bits
	CMD(CRT_CMD,8,0x00),	// row scan
	CMD(CRT_CMD,9,0x40),	// max scan line
	CMD(CRT_CMD,0x0A,0),	// cursor start
	CMD(CRT_CMD,0x0B,0),	// cursor end
	CMD(CRT_CMD,0x0C,0),	// start address (hi)
	CMD(CRT_CMD,0x0D,0),	// start address (lo)
	CMD(CRT_CMD,0x0E,0),	// cursor address
	CMD(CRT_CMD,0x0F,0),	// cursor address
	CMD(CRT_CMD,0x10,0xea),	// v retrace start
	CMD(CRT_CMD,0x11,0x8c),	// v retrace end
	CMD(CRT_CMD,0x12,0xdf),	// v disp enable end
	CMD(CRT_CMD,0x13,0x28),	// offset
	CMD(CRT_CMD,0x14,0x00),	// underline
	CMD(CRT_CMD,0x15,0xe7),	// v blank start
	CMD(CRT_CMD,0x16,0x04),	// v blank end
	CMD(CRT_CMD,0x17,0xe3),	// mode control
	CMD(CRT_CMD,0x18,0xff),	// line compare

	CMD(GRF_CMD,0x00,0x00),	// set/reset
	CMD(GRF_CMD,0x01,0x0f),	// enable set/reset
	CMD(GRF_CMD,0x02,0x00),	// color compare
	CMD(GRF_CMD,0x03,0x00),	// data rotate
	CMD(GRF_CMD,0x04,0x00),	// read map select
	CMD(GRF_CMD,0x05,0x20),	// write mode(3), even/odd(10), srmode(20), mode256(40) !40 works better than 60, 4 bits per pixel! 0 works too
	CMD(GRF_CMD,0x06,0x05),	// graphics(1), chain(2), mmapaddress(c)
	CMD(GRF_CMD,0x07,0x0f),	// color don't care
	CMD(GRF_CMD,0x08,0xff),	// bit mask

	CMD(ATR_CMD,0x00,0x00),	// default palette
	CMD(ATR_CMD,0x01,0x01),
	CMD(ATR_CMD,0x02,0x02),
	CMD(ATR_CMD,0x03,0x03),
	CMD(ATR_CMD,0x04,0x04),
	CMD(ATR_CMD,0x05,0x05),
	CMD(ATR_CMD,0x06,0x06),
	CMD(ATR_CMD,0x07,0x07),
	CMD(ATR_CMD,0x08,0x08),
	CMD(ATR_CMD,0x09,0x09),
	CMD(ATR_CMD,0x0a,0x0a),
	CMD(ATR_CMD,0x0b,0x0b),
	CMD(ATR_CMD,0x0c,0x0c),
	CMD(ATR_CMD,0x0d,0x0d),
	CMD(ATR_CMD,0x0e,0x0e),
	CMD(ATR_CMD,0x0f,0x0f),
	CMD(ATR_CMD,0x10,0x01),	// graphics(1), mono-attr(2), blink(8)
	CMD(ATR_CMD,0x11,0x00),	// overscan color
	CMD(ATR_CMD,0x12,0x0f),	// plane enable
	CMD(ATR_CMD,0x13,0x00),	// h pixel pan

	CMD(EXT_CMD,0x10,0x10),	// ext start address
	CMD(EXT_CMD,0x11,0x00),	// crt overflow
//	CMD(EXT_CMD,0x12,0x00),	// enable irq(1)
//	CMD(EXT_CMD,0x13,0x00),
	CMD(EXT_CMD,0x14,0x51),	// vga mem fetches per row
	CMD(EXT_CMD,0x15,0x00),
//	CMD(EXT_CMD,0x30,0x21),	// zero wait(4), PCI burst(20), linear size (3)
//	CMD(EXT_CMD,0x31,0x00),	// ext seg write
//	CMD(EXT_CMD,0x32,0x00),	// ext seg read
//	CMD(EXT_CMD,0x33,0x01),	// linear enable(1), segment on(10), 
//	CMD(EXT_CMD,0x50,0x00),	// sprite control
//	CMD(EXT_CMD,0x51,0x00),
//	CMD(EXT_CMD,0x52,0x00),
//	CMD(EXT_CMD,0x53,0x00),
//	CMD(EXT_CMD,0x54,0x00),
//	CMD(EXT_CMD,0x55,0x00),
//	CMD(EXT_CMD,0x56,0x00),	// sprite cursor, DAC access (4)
	CMD(EXT_CMD,0x57,0x01),	// ext graphic mode(1)	!must be set
//	CMD(EXT_CMD,0x58,0x00),	// RAMDAC
//	CMD(EXT_CMD,0x59,0x00),
//	CMD(EXT_CMD,0x5a,0x00),
////	CMD(EXT_CMD,0x70,0x0b),	// slow mem(1), ras control(3e), 
////	CMD(EXT_CMD,0x71,0x10),	// mem arch(3), sync mem(4), vram(8), mem-type(e0)=60
////	CMD(EXT_CMD,0x72,0x45),	// mem size(3), bus wid(4), refresh(c0)
	CMD(EXT_CMD,0x73,0x30),	// fifo over(81), fifosize(8)
	CMD(EXT_CMD,0x74,0x1b),	// fifo size
	CMD(EXT_CMD,0x75,0x1e),	// fifo size
	CMD(EXT_CMD,0x76,0x00),	// eeprom 
	CMD(EXT_CMD,0x77,0x00),	// color depth(f): 256=1, 32k=6, 64k=2, 16M=4, 4G=3, 256l=9
//	CMD(EXT_CMD,0x78,0x00),	
//	CMD(EXT_CMD,0x79,0x00),
//	CMD(EXT_CMD,0x7a,0xc8),	// cas control
	CMD(EXT_CMD,0x90,0x01),	// ext graphics mode	!must be set
	CMD(EXT_CMD,0xb0,0xd2),	// vclock numerator
	CMD(EXT_CMD,0xb1,0xce),	// vclock denom
//	CMD(EXT_CMD,0xb2,0xdb),	// memclock numerator
//	CMD(EXT_CMD,0xb3,0x54),	// memclock denom
	CMD(EXT_CMD,0xb9,0x80),	// toggle the clock pll params
	CMD(EXT_CMD,0xb9,0x00),

	CMD(PAL_CMD,0x00,0x04),	// 10=bypass the DAC

};

#if 0
// we should not be using 75Hz for 640x480 modes, as TV_out will not work...
//75 Hz
const int a640x480x8[] = {
	CMD(MISC_CMD,0x00,0xef), // sync polarity(c0), clock select(0c), ram-enable(2), color(1)

	CMD(SEQ_CMD,0x00,0x01),	// sync & async reset: 0 to reset
	CMD(SEQ_CMD,0x01,0x01),	// clock mode, screen-off(20)
	CMD(SEQ_CMD,0x02,0x0f), // write mask
	CMD(SEQ_CMD,0x03,0x00), // font select
	CMD(SEQ_CMD,0x04,0x0e), // ext mem(2), chain4(8)
	CMD(SEQ_CMD,0x00,0x03),	// sync & async reset: 0 to reset

	CMD(CRT_CMD,0x11,0x0b),	// unlock CR0-7
	CMD(CRT_CMD,0,0x64),	// h total
	CMD(CRT_CMD,1,0x4F),	// h disp end
	CMD(CRT_CMD,2,0x50),	// h blank start
	CMD(CRT_CMD,3,0x85),	// h blank end
	CMD(CRT_CMD,4,0x53),	// h retrace start
	CMD(CRT_CMD,5,0x81),	// h retrace end
	CMD(CRT_CMD,6,0xF2),	// v total
	CMD(CRT_CMD,7,0x1F),	// extra bits
	CMD(CRT_CMD,8,0x00),	// row scan
	CMD(CRT_CMD,9,0x40),	// max scan line
	CMD(CRT_CMD,0x0A,0),	// cursor start
	CMD(CRT_CMD,0x0B,0),	// cursor end
	CMD(CRT_CMD,0x0C,0),	// start address (hi)
	CMD(CRT_CMD,0x0D,0),	// start address (lo)
	CMD(CRT_CMD,0x0E,0),	// cursor address
	CMD(CRT_CMD,0x0F,0),	// cursor address
	CMD(CRT_CMD,0x10,0xE2),	// v retrace start
	CMD(CRT_CMD,0x11,0x87),	// v retrace end
	CMD(CRT_CMD,0x12,0xDF),	// v disp enable end
	CMD(CRT_CMD,0x13,0x50),	// offset
	CMD(CRT_CMD,0x14,0x00),	// underline
	CMD(CRT_CMD,0x15,0xE0),	// v blank start
	CMD(CRT_CMD,0x16,0xF1),	// v blank end
	CMD(CRT_CMD,0x17,0xC3),	// mode control
	CMD(CRT_CMD,0x18,0xFF),	// line compare

	CMD(GRF_CMD,0x00,0x00),	// set/reset
	CMD(GRF_CMD,0x01,0x00),	// enable set/reset
	CMD(GRF_CMD,0x02,0x00),	// color compare
	CMD(GRF_CMD,0x03,0x00),	// data rotate
	CMD(GRF_CMD,0x04,0x00),	// read map select
	CMD(GRF_CMD,0x05,0x60),	// write mode(3), even/odd(10), srmode(20), mode256(40)
	CMD(GRF_CMD,0x06,0x05),	// graphics(1), chain(2), mmapaddress(c)
	CMD(GRF_CMD,0x07,0x0f),	// color don't care
	CMD(GRF_CMD,0x08,0xff),	// bit mask

	CMD(ATR_CMD,0x00,0x00),	// default palette
	CMD(ATR_CMD,0x01,0x01),
	CMD(ATR_CMD,0x02,0x02),
	CMD(ATR_CMD,0x03,0x03),
	CMD(ATR_CMD,0x04,0x04),
	CMD(ATR_CMD,0x05,0x05),
	CMD(ATR_CMD,0x06,0x06),
	CMD(ATR_CMD,0x07,0x07),
	CMD(ATR_CMD,0x08,0x08),
	CMD(ATR_CMD,0x09,0x09),
	CMD(ATR_CMD,0x0a,0x0a),
	CMD(ATR_CMD,0x0b,0x0b),
	CMD(ATR_CMD,0x0c,0x0c),
	CMD(ATR_CMD,0x0d,0x0d),
	CMD(ATR_CMD,0x0e,0x0e),
	CMD(ATR_CMD,0x0f,0x0f),
	CMD(ATR_CMD,0x10,0x01),	// graphics(1), mono-attr(2), blink(8)
	CMD(ATR_CMD,0x11,0x00),	// overscan color
	CMD(ATR_CMD,0x12,0x0f),	// plane enable
	CMD(ATR_CMD,0x13,0x00),	// h pixel pan

	CMD(EXT_CMD,0x10,0x10),	// ext start address
	CMD(EXT_CMD,0x11,0x00),	// crt overflow
	CMD(EXT_CMD,0x12,0x00),	// enable irq(1)
	CMD(EXT_CMD,0x13,0x00),
	CMD(EXT_CMD,0x14,0x51),	// vga mem fetches per row
	CMD(EXT_CMD,0x15,0x00),
//	CMD(EXT_CMD,0x30,0x21),	// zero wait(4), PCI burst(20), linear size (3)
	CMD(EXT_CMD,0x31,0x00),	// ext seg write
	CMD(EXT_CMD,0x32,0x00),	// ext seg read
	CMD(EXT_CMD,0x33,0x01),	// linear enable(1), segment on(10), 
	CMD(EXT_CMD,0x50,0x00),	// sprite control
	CMD(EXT_CMD,0x51,0x00),
	CMD(EXT_CMD,0x52,0x00),
	CMD(EXT_CMD,0x53,0x00),
	CMD(EXT_CMD,0x54,0x00),
	CMD(EXT_CMD,0x55,0x00),
	CMD(EXT_CMD,0x56,0x00),	// sprite cursor, DAC access (4)
	CMD(EXT_CMD,0x57,0x01),	// ext graphic mode(1)
	CMD(EXT_CMD,0x58,0x00),	// RAMDAC
	CMD(EXT_CMD,0x59,0x00),
	CMD(EXT_CMD,0x5a,0x00),
//	CMD(EXT_CMD,0x70,0x0B),	// slow mem(1), ras control(3e), 
//	CMD(EXT_CMD,0x71,0x10),	// mem arch(3), sync mem(4), vram(8), mem-type(e0)=60
//	CMD(EXT_CMD,0x72,0x45),	// mem size(3), bus wid(4), refresh(c0)
	CMD(EXT_CMD,0x73,0x30),	// fifo over(81), fifosize(8)
	CMD(EXT_CMD,0x74,0x1b),	// fifo size
	CMD(EXT_CMD,0x75,0x1e),	// fifo size
	CMD(EXT_CMD,0x76,0x00),	// eeprom 
	CMD(EXT_CMD,0x77,0x01),	// color depth(f): 256=1, 32k=6, 64k=2, 16M=4, 4G=3, 256l=9
	CMD(EXT_CMD,0x7a,0xc8),	// cas control
	CMD(EXT_CMD,0xb0,0x2B),	// vclock numerator
	CMD(EXT_CMD,0xb1,0x84),	// vclock denom
	CMD(EXT_CMD,0xb2,0xDB),	// memclock numerator
	CMD(EXT_CMD,0xb3,0x54),	// memclock denom
	CMD(EXT_CMD,0x90,0x01),	// graphics mode
	CMD(EXT_CMD,0xb9,0x80),	// toggle the clock pll params
	CMD(EXT_CMD,0xb9,0x00),

	CMD(PAL_CMD,0x00,0x14),	// 10=bypass the DAC

};

//75 Hz
const int a640x480x16[] = {
	CMD(MISC_CMD,0x00,0xef), // sync polarity(c0), clock select(0c), ram-enable(2), color(1)

	CMD(SEQ_CMD,0x00,0x01),	// sync & async reset: 0 to reset
	CMD(SEQ_CMD,0x01,0x01),	// clock mode, screen-off(20)
	CMD(SEQ_CMD,0x02,0x0f), // write mask
	CMD(SEQ_CMD,0x03,0x00), // font select
	CMD(SEQ_CMD,0x04,0x0e), // ext mem(2), chain4(8)
	CMD(SEQ_CMD,0x00,0x03),	// sync & async reset: 0 to reset

	CMD(CRT_CMD,0x11,0x0b),	// unlock CR0-7
	CMD(CRT_CMD,0,0x64),	// h total
	CMD(CRT_CMD,1,0x4F),	// h disp end
	CMD(CRT_CMD,2,0x50),	// h blank start
	CMD(CRT_CMD,3,0x85),	// h blank end
	CMD(CRT_CMD,4,0x53),	// h retrace start
	CMD(CRT_CMD,5,0x81),	// h retrace end
	CMD(CRT_CMD,6,0xF2),	// v total
	CMD(CRT_CMD,7,0x1F),	// extra bits
	CMD(CRT_CMD,8,0x00),	// row scan
	CMD(CRT_CMD,9,0x40),	// max scan line
	CMD(CRT_CMD,0x0A,0),	// cursor start
	CMD(CRT_CMD,0x0B,0),	// cursor end
	CMD(CRT_CMD,0x0C,0),	// start address (hi)
	CMD(CRT_CMD,0x0D,0),	// start address (lo)
	CMD(CRT_CMD,0x0E,0),	// cursor address
	CMD(CRT_CMD,0x0F,0),	// cursor address
	CMD(CRT_CMD,0x10,0xE2),	// v retrace start
	CMD(CRT_CMD,0x11,0x87),	// v retrace end
	CMD(CRT_CMD,0x12,0xDF),	// v disp enable end
	CMD(CRT_CMD,0x13,0xA0),	// offset
	CMD(CRT_CMD,0x14,0x00),	// underline
	CMD(CRT_CMD,0x15,0xE0),	// v blank start
	CMD(CRT_CMD,0x16,0xF1),	// v blank end
	CMD(CRT_CMD,0x17,0xC3),	// mode control
	CMD(CRT_CMD,0x18,0xFF),	// line compare

	CMD(GRF_CMD,0x00,0x00),	// set/reset
	CMD(GRF_CMD,0x01,0x00),	// enable set/reset
	CMD(GRF_CMD,0x02,0x00),	// color compare
	CMD(GRF_CMD,0x03,0x00),	// data rotate
	CMD(GRF_CMD,0x04,0x00),	// read map select
	CMD(GRF_CMD,0x05,0x60),	// write mode(3), even/odd(10), srmode(20), mode256(40)
	CMD(GRF_CMD,0x06,0x05),	// graphics(1), chain(2), mmapaddress(c)
	CMD(GRF_CMD,0x07,0x0f),	// color don't care
	CMD(GRF_CMD,0x08,0xff),	// bit mask

	CMD(ATR_CMD,0x00,0x00),	// default palette
	CMD(ATR_CMD,0x01,0x01),
	CMD(ATR_CMD,0x02,0x02),
	CMD(ATR_CMD,0x03,0x03),
	CMD(ATR_CMD,0x04,0x04),
	CMD(ATR_CMD,0x05,0x05),
	CMD(ATR_CMD,0x06,0x06),
	CMD(ATR_CMD,0x07,0x07),
	CMD(ATR_CMD,0x08,0x08),
	CMD(ATR_CMD,0x09,0x09),
	CMD(ATR_CMD,0x0a,0x0a),
	CMD(ATR_CMD,0x0b,0x0b),
	CMD(ATR_CMD,0x0c,0x0c),
	CMD(ATR_CMD,0x0d,0x0d),
	CMD(ATR_CMD,0x0e,0x0e),
	CMD(ATR_CMD,0x0f,0x0f),
	CMD(ATR_CMD,0x10,0x01),	// graphics(1), mono-attr(2), blink(8)
	CMD(ATR_CMD,0x11,0x00),	// overscan color
	CMD(ATR_CMD,0x12,0x0f),	// plane enable
	CMD(ATR_CMD,0x13,0x00),	// h pixel pan

	CMD(EXT_CMD,0x10,0x10),	// ext start address
	CMD(EXT_CMD,0x11,0x00),	// crt overflow
	CMD(EXT_CMD,0x12,0x00),	// enable irq(1)
	CMD(EXT_CMD,0x13,0x00),
	CMD(EXT_CMD,0x14,0xA1),	// vga mem fetches per row
	CMD(EXT_CMD,0x15,0x00),
//	CMD(EXT_CMD,0x30,0x21),	// zero wait(4), PCI burst(20), linear size (3)
	CMD(EXT_CMD,0x31,0x00),	// ext seg write
	CMD(EXT_CMD,0x32,0x00),	// ext seg read
	CMD(EXT_CMD,0x33,0x01),	// linear enable(1), segment on(10), 
	CMD(EXT_CMD,0x50,0x00),	// sprite control
	CMD(EXT_CMD,0x51,0x00),
	CMD(EXT_CMD,0x52,0x00),
	CMD(EXT_CMD,0x53,0x00),
	CMD(EXT_CMD,0x54,0x00),
	CMD(EXT_CMD,0x55,0x00),
	CMD(EXT_CMD,0x56,0x00),	// sprite cursor, DAC access (4)
	CMD(EXT_CMD,0x57,0x01),	// ext graphic mode(1)
	CMD(EXT_CMD,0x58,0x00),	// RAMDAC
	CMD(EXT_CMD,0x59,0x00),
	CMD(EXT_CMD,0x5a,0x00),
//	CMD(EXT_CMD,0x70,0x0B),	// slow mem(1), ras control(3e), 
//	CMD(EXT_CMD,0x71,0x10),	// mem arch(3), sync mem(4), vram(8), mem-type(e0)=60
//	CMD(EXT_CMD,0x72,0x45),	// mem size(3), bus wid(4), refresh(c0)
	CMD(EXT_CMD,0x73,0x30),	// fifo over(81), fifosize(8)
	CMD(EXT_CMD,0x74,0x1b),	// fifo size
	CMD(EXT_CMD,0x75,0x1e),	// fifo size
	CMD(EXT_CMD,0x76,0x00),	// eeprom 
	CMD(EXT_CMD,0x77,0x02),	// color depth(f): 256=1, 32k=6, 64k=2, 16M=4, 4G=3, 256l=9
	CMD(EXT_CMD,0x7a,0xc8),	// cas control
	CMD(EXT_CMD,0xb0,0x2B),	// vclock numerator
	CMD(EXT_CMD,0xb1,0x84),	// vclock denom
	CMD(EXT_CMD,0xb2,0xDB),	// memclock numerator
	CMD(EXT_CMD,0xb3,0x54),	// memclock denom
	CMD(EXT_CMD,0x90,0x01),	// graphics mode
	CMD(EXT_CMD,0xb9,0x80),	// toggle the clock pll params
	CMD(EXT_CMD,0xb9,0x00),

	CMD(PAL_CMD,0x00,0x14),	// 10=bypass the DAC

};


//75 Hz
const int a640x480x24[] = {
	CMD(MISC_CMD,0x00,0xe3), // sync polarity(c0), clock select(0c), ram-enable(2), color(1)

	CMD(SEQ_CMD,0x00,0x01),	// sync & async reset: 0 to reset
	CMD(SEQ_CMD,0x01,0x01),	// clock mode, screen-off(20)
	CMD(SEQ_CMD,0x02,0x0f), // write mask
	CMD(SEQ_CMD,0x03,0x00), // font select
	CMD(SEQ_CMD,0x04,0x0e), // ext mem(2), chain4(8)
	CMD(SEQ_CMD,0x00,0x03),	// sync & async reset: 0 to reset

	CMD(CRT_CMD,0x11,0x0b),	// unlock CR0-7
	CMD(CRT_CMD,0,0x64),	// h total
	CMD(CRT_CMD,1,0x4F),	// h disp end
	CMD(CRT_CMD,2,0x50),	// h blank start
	CMD(CRT_CMD,3,0x85),	// h blank end
	CMD(CRT_CMD,4,0x53),	// h retrace start
	CMD(CRT_CMD,5,0x81),	// h retrace end
	CMD(CRT_CMD,6,0xF2),	// v total
	CMD(CRT_CMD,7,0x1F),	// extra bits
	CMD(CRT_CMD,8,0x00),	// row scan
	CMD(CRT_CMD,9,0x40),	// max scan line
	CMD(CRT_CMD,0x0A,0),	// cursor start
	CMD(CRT_CMD,0x0B,0),	// cursor end
	CMD(CRT_CMD,0x0C,0),	// start address (hi)
	CMD(CRT_CMD,0x0D,0),	// start address (lo)
	CMD(CRT_CMD,0x0E,0),	// cursor address
	CMD(CRT_CMD,0x0F,0),	// cursor address
	CMD(CRT_CMD,0x10,0xE2),	// v retrace start
	CMD(CRT_CMD,0x11,0x87),	// v retrace end
	CMD(CRT_CMD,0x12,0xDF),	// v disp enable end
	CMD(CRT_CMD,0x13,0xF0),	// offset
	CMD(CRT_CMD,0x14,0x00),	// underline
	CMD(CRT_CMD,0x15,0xE0),	// v blank start
	CMD(CRT_CMD,0x16,0xF1),	// v blank end
	CMD(CRT_CMD,0x17,0xC3),	// mode control
	CMD(CRT_CMD,0x18,0xFF),	// line compare

	CMD(GRF_CMD,0x00,0x00),	// set/reset
	CMD(GRF_CMD,0x01,0x00),	// enable set/reset
	CMD(GRF_CMD,0x02,0x00),	// color compare
	CMD(GRF_CMD,0x03,0x00),	// data rotate
	CMD(GRF_CMD,0x04,0x00),	// read map select
	CMD(GRF_CMD,0x05,0x60),	// write mode(3), even/odd(10), srmode(20), mode256(40)
	CMD(GRF_CMD,0x06,0x05),	// graphics(1), chain(2), mmapaddress(c)
	CMD(GRF_CMD,0x07,0x0f),	// color don't care
	CMD(GRF_CMD,0x08,0xff),	// bit mask

	CMD(ATR_CMD,0x00,0x00),	// default palette
	CMD(ATR_CMD,0x01,0x01),
	CMD(ATR_CMD,0x02,0x02),
	CMD(ATR_CMD,0x03,0x03),
	CMD(ATR_CMD,0x04,0x04),
	CMD(ATR_CMD,0x05,0x05),
	CMD(ATR_CMD,0x06,0x06),
	CMD(ATR_CMD,0x07,0x07),
	CMD(ATR_CMD,0x08,0x08),
	CMD(ATR_CMD,0x09,0x09),
	CMD(ATR_CMD,0x0a,0x0a),
	CMD(ATR_CMD,0x0b,0x0b),
	CMD(ATR_CMD,0x0c,0x0c),
	CMD(ATR_CMD,0x0d,0x0d),
	CMD(ATR_CMD,0x0e,0x0e),
	CMD(ATR_CMD,0x0f,0x0f),
	CMD(ATR_CMD,0x10,0x01),	// graphics(1), mono-attr(2), blink(8)
	CMD(ATR_CMD,0x11,0x00),	// overscan color
	CMD(ATR_CMD,0x12,0x0f),	// plane enable
	CMD(ATR_CMD,0x13,0x00),	// h pixel pan

	CMD(EXT_CMD,0x10,0x10),	// ext start address
	CMD(EXT_CMD,0x11,0x00),	// crt overflow
	CMD(EXT_CMD,0x12,0x00),	// enable irq(1)
	CMD(EXT_CMD,0x13,0x00),
	CMD(EXT_CMD,0x14,0xf1),	// vga mem fetches per row
	CMD(EXT_CMD,0x15,0x00),
//	CMD(EXT_CMD,0x30,0x21),	// zero wait(4), PCI burst(20), linear size (3)
	CMD(EXT_CMD,0x31,0x00),	// ext seg write
	CMD(EXT_CMD,0x32,0x00),	// ext seg read
	CMD(EXT_CMD,0x33,0x01),	// linear enable(1), segment on(10), 
	CMD(EXT_CMD,0x50,0x00),	// sprite control
	CMD(EXT_CMD,0x51,0x00),
	CMD(EXT_CMD,0x52,0x00),
	CMD(EXT_CMD,0x53,0x00),
	CMD(EXT_CMD,0x54,0x00),
	CMD(EXT_CMD,0x55,0x00),
	CMD(EXT_CMD,0x56,0x00),	// sprite cursor, DAC access (4)
	CMD(EXT_CMD,0x57,0x01),	// ext graphic mode(1)
	CMD(EXT_CMD,0x58,0x00),	// RAMDAC
	CMD(EXT_CMD,0x59,0x00),
	CMD(EXT_CMD,0x5a,0x00),
//	CMD(EXT_CMD,0x70,0x0B),	// slow mem(1), ras control(3e), 
//	CMD(EXT_CMD,0x71,0x10),	// mem arch(3), sync mem(4), vram(8), mem-type(e0)=60
//	CMD(EXT_CMD,0x72,0x45),	// mem size(3), bus wid(4), refresh(c0)
	CMD(EXT_CMD,0x73,0x30),	// fifo over(81), fifosize(8)
	CMD(EXT_CMD,0x74,0x1b),	// fifo size
	CMD(EXT_CMD,0x75,0x1e),	// fifo size
	CMD(EXT_CMD,0x76,0x00),	// eeprom 
	CMD(EXT_CMD,0x77,0x04),	// color depth(f): 256=1, 32k=6, 64k=2, 16M=4, 4G=3, 256l=9
	CMD(EXT_CMD,0x78,0x00),	
	CMD(EXT_CMD,0x79,0x00),
	CMD(EXT_CMD,0x7a,0xc8),	// cas control
	CMD(EXT_CMD,0xb0,0x2B),	// vclock numerator
	CMD(EXT_CMD,0xb1,0x84),	// vclock denom
	CMD(EXT_CMD,0xb2,0xDB),	// memclock numerator
	CMD(EXT_CMD,0xb3,0x54),	// memclock denom
	CMD(EXT_CMD,0x90,0x01),	// graphics mode
	CMD(EXT_CMD,0xb9,0x80),	// toggle the clock pll params
	CMD(EXT_CMD,0xb9,0x00),

	CMD(PAL_CMD,0x00,0x14),	// 10=bypass the DAC

};
#endif

//60 Hz
const int a640x480x8[] = {
	CMD(MISC_CMD,0x00,0xef), // sync polarity(c0), clock select(0c), ram-enable(2), color(1)

	CMD(SEQ_CMD,0x00,0x01),	// sync & async reset: 0 to reset
	CMD(SEQ_CMD,0x01,0x01),	// clock mode, screen-off(20)
	CMD(SEQ_CMD,0x02,0x0f), // write mask
	CMD(SEQ_CMD,0x03,0x00), // font select
	CMD(SEQ_CMD,0x04,0x0e), // ext mem(2), chain4(8)
	CMD(SEQ_CMD,0x00,0x03),	// sync & async reset: 0 to reset

	CMD(CRT_CMD,0x11,0x0b),	// unlock CR0-7
	CMD(CRT_CMD,0,0x5F),	// h total
	CMD(CRT_CMD,1,0x4F),	// h disp end
	CMD(CRT_CMD,2,0x50),	// h blank start
	CMD(CRT_CMD,3,0x80),	// h blank end
	CMD(CRT_CMD,4,0x52),	// h retrace start
	CMD(CRT_CMD,5,0x9D),	// h retrace end
	CMD(CRT_CMD,6,0x0B),	// v total
	CMD(CRT_CMD,7,0x3E),	// extra bits
	CMD(CRT_CMD,8,0x00),	// row scan
	CMD(CRT_CMD,9,0x40),	// max scan line
	CMD(CRT_CMD,0x0A,0),	// cursor start
	CMD(CRT_CMD,0x0B,0),	// cursor end
	CMD(CRT_CMD,0x0C,0),	// start address (hi)
	CMD(CRT_CMD,0x0D,0),	// start address (lo)
	CMD(CRT_CMD,0x0E,0),	// cursor address
	CMD(CRT_CMD,0x0F,0),	// cursor address
	CMD(CRT_CMD,0x10,0xE9),	// v retrace start
	CMD(CRT_CMD,0x11,0x8B),	// v retrace end
	CMD(CRT_CMD,0x12,0xDF),	// v disp enable end
	CMD(CRT_CMD,0x13,0x50),	// offset
	CMD(CRT_CMD,0x14,0x00),	// underline
	CMD(CRT_CMD,0x15,0xE6),	// v blank start
	CMD(CRT_CMD,0x16,0x04),	// v blank end
	CMD(CRT_CMD,0x17,0xC3),	// mode control
	CMD(CRT_CMD,0x18,0xFF),	// line compare

	CMD(GRF_CMD,0x00,0x00),	// set/reset
	CMD(GRF_CMD,0x01,0x00),	// enable set/reset
	CMD(GRF_CMD,0x02,0x00),	// color compare
	CMD(GRF_CMD,0x03,0x00),	// data rotate
	CMD(GRF_CMD,0x04,0x00),	// read map select
	CMD(GRF_CMD,0x05,0x60),	// write mode(3), even/odd(10), srmode(20), mode256(40)
	CMD(GRF_CMD,0x06,0x05),	// graphics(1), chain(2), mmapaddress(c)
	CMD(GRF_CMD,0x07,0x0f),	// color don't care
	CMD(GRF_CMD,0x08,0xff),	// bit mask

	CMD(ATR_CMD,0x00,0x00),	// default palette
	CMD(ATR_CMD,0x01,0x01),
	CMD(ATR_CMD,0x02,0x02),
	CMD(ATR_CMD,0x03,0x03),
	CMD(ATR_CMD,0x04,0x04),
	CMD(ATR_CMD,0x05,0x05),
	CMD(ATR_CMD,0x06,0x06),
	CMD(ATR_CMD,0x07,0x07),
	CMD(ATR_CMD,0x08,0x08),
	CMD(ATR_CMD,0x09,0x09),
	CMD(ATR_CMD,0x0a,0x0a),
	CMD(ATR_CMD,0x0b,0x0b),
	CMD(ATR_CMD,0x0c,0x0c),
	CMD(ATR_CMD,0x0d,0x0d),
	CMD(ATR_CMD,0x0e,0x0e),
	CMD(ATR_CMD,0x0f,0x0f),
	CMD(ATR_CMD,0x10,0x01),	// graphics(1), mono-attr(2), blink(8)
	CMD(ATR_CMD,0x11,0x00),	// overscan color
	CMD(ATR_CMD,0x12,0x0f),	// plane enable
	CMD(ATR_CMD,0x13,0x00),	// h pixel pan

	CMD(EXT_CMD,0x10,0x10),	// ext start address
	CMD(EXT_CMD,0x11,0x00),	// crt overflow
	CMD(EXT_CMD,0x12,0x00),	// enable irq(1)
	CMD(EXT_CMD,0x13,0x00),
	CMD(EXT_CMD,0x14,0x51),	// vga mem fetches per row
	CMD(EXT_CMD,0x15,0x00),
//	CMD(EXT_CMD,0x30,0x21),	// zero wait(4), PCI burst(20), linear size (3)
	CMD(EXT_CMD,0x31,0x00),	// ext seg write
	CMD(EXT_CMD,0x32,0x00),	// ext seg read
	CMD(EXT_CMD,0x33,0x01),	// linear enable(1), segment on(10), 
	CMD(EXT_CMD,0x50,0x00),	// sprite control
	CMD(EXT_CMD,0x51,0x00),
	CMD(EXT_CMD,0x52,0x00),
	CMD(EXT_CMD,0x53,0x00),
	CMD(EXT_CMD,0x54,0x00),
	CMD(EXT_CMD,0x55,0x00),
	CMD(EXT_CMD,0x56,0x00),	// sprite cursor, DAC access (4)
	CMD(EXT_CMD,0x57,0x01),	// ext graphic mode(1)
	CMD(EXT_CMD,0x58,0x00),	// RAMDAC
	CMD(EXT_CMD,0x59,0x00),
	CMD(EXT_CMD,0x5a,0x00),
//	CMD(EXT_CMD,0x70,0x0B),	// slow mem(1), ras control(3e), 
//	CMD(EXT_CMD,0x71,0x10),	// mem arch(3), sync mem(4), vram(8), mem-type(e0)=60
//	CMD(EXT_CMD,0x72,0x45),	// mem size(3), bus wid(4), refresh(c0)
	CMD(EXT_CMD,0x73,0x30),	// fifo over(81), fifosize(8)
	CMD(EXT_CMD,0x74,0x1b),	// fifo size
	CMD(EXT_CMD,0x75,0x1e),	// fifo size
	CMD(EXT_CMD,0x76,0x00),	// eeprom 
	CMD(EXT_CMD,0x77,0x01),	// color depth(f): 256=1, 32k=6, 64k=2, 16M=4, 4G=3, 256l=9
	CMD(EXT_CMD,0x7a,0xc8),	// cas control
	CMD(EXT_CMD,0xb0,0xD2),	// vclock numerator
	CMD(EXT_CMD,0xb1,0xCE),	// vclock denom
	CMD(EXT_CMD,0xb2,0xDB),	// memclock numerator
	CMD(EXT_CMD,0xb3,0x54),	// memclock denom
	CMD(EXT_CMD,0x90,0x01),	// graphics mode
	CMD(EXT_CMD,0xb9,0x80),	// toggle the clock pll params
	CMD(EXT_CMD,0xb9,0x00),

	CMD(PAL_CMD,0x00,0x14),	// 10=bypass the DAC

};

//60 Hz
const int a640x480x16[] = {
	CMD(MISC_CMD,0x00,0xef), // sync polarity(c0), clock select(0c), ram-enable(2), color(1)

	CMD(SEQ_CMD,0x00,0x01),	// sync & async reset: 0 to reset
	CMD(SEQ_CMD,0x01,0x01),	// clock mode, screen-off(20)
	CMD(SEQ_CMD,0x02,0x0f), // write mask
	CMD(SEQ_CMD,0x03,0x00), // font select
	CMD(SEQ_CMD,0x04,0x0e), // ext mem(2), chain4(8)
	CMD(SEQ_CMD,0x00,0x03),	// sync & async reset: 0 to reset

	CMD(CRT_CMD,0x11,0x0b),	// unlock CR0-7
	CMD(CRT_CMD,0,0x5F),	// h total
	CMD(CRT_CMD,1,0x4F),	// h disp end
	CMD(CRT_CMD,2,0x50),	// h blank start
	CMD(CRT_CMD,3,0x80),	// h blank end
	CMD(CRT_CMD,4,0x52),	// h retrace start
	CMD(CRT_CMD,5,0x9D),	// h retrace end
	CMD(CRT_CMD,6,0x0B),	// v total
	CMD(CRT_CMD,7,0x3E),	// extra bits
	CMD(CRT_CMD,8,0x00),	// row scan
	CMD(CRT_CMD,9,0x40),	// max scan line
	CMD(CRT_CMD,0x0A,0),	// cursor start
	CMD(CRT_CMD,0x0B,0),	// cursor end
	CMD(CRT_CMD,0x0C,0),	// start address (hi)
	CMD(CRT_CMD,0x0D,0),	// start address (lo)
	CMD(CRT_CMD,0x0E,0),	// cursor address
	CMD(CRT_CMD,0x0F,0),	// cursor address
	CMD(CRT_CMD,0x10,0xE9),	// v retrace start
	CMD(CRT_CMD,0x11,0x8B),	// v retrace end
	CMD(CRT_CMD,0x12,0xDF),	// v disp enable end
	CMD(CRT_CMD,0x13,0xA0),	// offset
	CMD(CRT_CMD,0x14,0x00),	// underline
	CMD(CRT_CMD,0x15,0xE6),	// v blank start
	CMD(CRT_CMD,0x16,0x04),	// v blank end
	CMD(CRT_CMD,0x17,0xC3),	// mode control
	CMD(CRT_CMD,0x18,0xFF),	// line compare

	CMD(GRF_CMD,0x00,0x00),	// set/reset
	CMD(GRF_CMD,0x01,0x00),	// enable set/reset
	CMD(GRF_CMD,0x02,0x00),	// color compare
	CMD(GRF_CMD,0x03,0x00),	// data rotate
	CMD(GRF_CMD,0x04,0x00),	// read map select
	CMD(GRF_CMD,0x05,0x60),	// write mode(3), even/odd(10), srmode(20), mode256(40)
	CMD(GRF_CMD,0x06,0x05),	// graphics(1), chain(2), mmapaddress(c)
	CMD(GRF_CMD,0x07,0x0f),	// color don't care
	CMD(GRF_CMD,0x08,0xff),	// bit mask

	CMD(ATR_CMD,0x00,0x00),	// default palette
	CMD(ATR_CMD,0x01,0x01),
	CMD(ATR_CMD,0x02,0x02),
	CMD(ATR_CMD,0x03,0x03),
	CMD(ATR_CMD,0x04,0x04),
	CMD(ATR_CMD,0x05,0x05),
	CMD(ATR_CMD,0x06,0x06),
	CMD(ATR_CMD,0x07,0x07),
	CMD(ATR_CMD,0x08,0x08),
	CMD(ATR_CMD,0x09,0x09),
	CMD(ATR_CMD,0x0a,0x0a),
	CMD(ATR_CMD,0x0b,0x0b),
	CMD(ATR_CMD,0x0c,0x0c),
	CMD(ATR_CMD,0x0d,0x0d),
	CMD(ATR_CMD,0x0e,0x0e),
	CMD(ATR_CMD,0x0f,0x0f),
	CMD(ATR_CMD,0x10,0x01),	// graphics(1), mono-attr(2), blink(8)
	CMD(ATR_CMD,0x11,0x00),	// overscan color
	CMD(ATR_CMD,0x12,0x0f),	// plane enable
	CMD(ATR_CMD,0x13,0x00),	// h pixel pan

	CMD(EXT_CMD,0x10,0x10),	// ext start address
	CMD(EXT_CMD,0x11,0x00),	// crt overflow
	CMD(EXT_CMD,0x12,0x00),	// enable irq(1)
	CMD(EXT_CMD,0x13,0x00),
	CMD(EXT_CMD,0x14,0xA1),	// vga mem fetches per row
	CMD(EXT_CMD,0x15,0x00),
//	CMD(EXT_CMD,0x30,0x21),	// zero wait(4), PCI burst(20), linear size (3)
	CMD(EXT_CMD,0x31,0x00),	// ext seg write
	CMD(EXT_CMD,0x32,0x00),	// ext seg read
	CMD(EXT_CMD,0x33,0x01),	// linear enable(1), segment on(10), 
	CMD(EXT_CMD,0x50,0x00),	// sprite control
	CMD(EXT_CMD,0x51,0x00),
	CMD(EXT_CMD,0x52,0x00),
	CMD(EXT_CMD,0x53,0x00),
	CMD(EXT_CMD,0x54,0x00),
	CMD(EXT_CMD,0x55,0x00),
	CMD(EXT_CMD,0x56,0x00),	// sprite cursor, DAC access (4)
	CMD(EXT_CMD,0x57,0x01),	// ext graphic mode(1)
	CMD(EXT_CMD,0x58,0x00),	// RAMDAC
	CMD(EXT_CMD,0x59,0x00),
	CMD(EXT_CMD,0x5a,0x00),
//	CMD(EXT_CMD,0x70,0x0B),	// slow mem(1), ras control(3e), 
//	CMD(EXT_CMD,0x71,0x10),	// mem arch(3), sync mem(4), vram(8), mem-type(e0)=60
//	CMD(EXT_CMD,0x72,0x45),	// mem size(3), bus wid(4), refresh(c0)
	CMD(EXT_CMD,0x73,0x30),	// fifo over(81), fifosize(8)
	CMD(EXT_CMD,0x74,0x1b),	// fifo size
	CMD(EXT_CMD,0x75,0x1e),	// fifo size
	CMD(EXT_CMD,0x76,0x00),	// eeprom 
	CMD(EXT_CMD,0x77,0x02),	// color depth(f): 256=1, 32k=6, 64k=2, 16M=4, 4G=3, 256l=9
	CMD(EXT_CMD,0x7a,0xc8),	// cas control
	CMD(EXT_CMD,0xb0,0xD2),	// vclock numerator
	CMD(EXT_CMD,0xb1,0xCE),	// vclock denom
	CMD(EXT_CMD,0xb2,0xDB),	// memclock numerator
	CMD(EXT_CMD,0xb3,0x54),	// memclock denom
	CMD(EXT_CMD,0x90,0x01),	// graphics mode
	CMD(EXT_CMD,0xb9,0x80),	// toggle the clock pll params
	CMD(EXT_CMD,0xb9,0x00),

	CMD(PAL_CMD,0x00,0x14),	// 10=bypass the DAC

};


//60 Hz
const int a640x480x24[] = {
	CMD(MISC_CMD,0x00,0xe3), // sync polarity(c0), clock select(0c), ram-enable(2), color(1)

	CMD(SEQ_CMD,0x00,0x01),	// sync & async reset: 0 to reset
	CMD(SEQ_CMD,0x01,0x01),	// clock mode, screen-off(20)
	CMD(SEQ_CMD,0x02,0x0f), // write mask
	CMD(SEQ_CMD,0x03,0x00), // font select
	CMD(SEQ_CMD,0x04,0x0e), // ext mem(2), chain4(8)
	CMD(SEQ_CMD,0x00,0x03),	// sync & async reset: 0 to reset

	CMD(CRT_CMD,0x11,0x0b),	// unlock CR0-7
	CMD(CRT_CMD,0,0x5F),	// h total
	CMD(CRT_CMD,1,0x4F),	// h disp end
	CMD(CRT_CMD,2,0x50),	// h blank start
	CMD(CRT_CMD,3,0x80),	// h blank end
	CMD(CRT_CMD,4,0x52),	// h retrace start
	CMD(CRT_CMD,5,0x9D),	// h retrace end
	CMD(CRT_CMD,6,0x0B),	// v total
	CMD(CRT_CMD,7,0x3E),	// extra bits
	CMD(CRT_CMD,8,0x00),	// row scan
	CMD(CRT_CMD,9,0x40),	// max scan line
	CMD(CRT_CMD,0x0A,0),	// cursor start
	CMD(CRT_CMD,0x0B,0),	// cursor end
	CMD(CRT_CMD,0x0C,0),	// start address (hi)
	CMD(CRT_CMD,0x0D,0),	// start address (lo)
	CMD(CRT_CMD,0x0E,0),	// cursor address
	CMD(CRT_CMD,0x0F,0),	// cursor address
	CMD(CRT_CMD,0x10,0xE9),	// v retrace start
	CMD(CRT_CMD,0x11,0x8B),	// v retrace end
	CMD(CRT_CMD,0x12,0xDF),	// v disp enable end
	CMD(CRT_CMD,0x13,0xF0),	// offset
	CMD(CRT_CMD,0x14,0x00),	// underline
	CMD(CRT_CMD,0x15,0xE6),	// v blank start
	CMD(CRT_CMD,0x16,0x04),	// v blank end
	CMD(CRT_CMD,0x17,0xC3),	// mode control
	CMD(CRT_CMD,0x18,0xFF),	// line compare

	CMD(GRF_CMD,0x00,0x00),	// set/reset
	CMD(GRF_CMD,0x01,0x00),	// enable set/reset
	CMD(GRF_CMD,0x02,0x00),	// color compare
	CMD(GRF_CMD,0x03,0x00),	// data rotate
	CMD(GRF_CMD,0x04,0x00),	// read map select
	CMD(GRF_CMD,0x05,0x60),	// write mode(3), even/odd(10), srmode(20), mode256(40)
	CMD(GRF_CMD,0x06,0x05),	// graphics(1), chain(2), mmapaddress(c)
	CMD(GRF_CMD,0x07,0x0f),	// color don't care
	CMD(GRF_CMD,0x08,0xff),	// bit mask

	CMD(ATR_CMD,0x00,0x00),	// default palette
	CMD(ATR_CMD,0x01,0x01),
	CMD(ATR_CMD,0x02,0x02),
	CMD(ATR_CMD,0x03,0x03),
	CMD(ATR_CMD,0x04,0x04),
	CMD(ATR_CMD,0x05,0x05),
	CMD(ATR_CMD,0x06,0x06),
	CMD(ATR_CMD,0x07,0x07),
	CMD(ATR_CMD,0x08,0x08),
	CMD(ATR_CMD,0x09,0x09),
	CMD(ATR_CMD,0x0a,0x0a),
	CMD(ATR_CMD,0x0b,0x0b),
	CMD(ATR_CMD,0x0c,0x0c),
	CMD(ATR_CMD,0x0d,0x0d),
	CMD(ATR_CMD,0x0e,0x0e),
	CMD(ATR_CMD,0x0f,0x0f),
	CMD(ATR_CMD,0x10,0x01),	// graphics(1), mono-attr(2), blink(8)
	CMD(ATR_CMD,0x11,0x00),	// overscan color
	CMD(ATR_CMD,0x12,0x0f),	// plane enable
	CMD(ATR_CMD,0x13,0x00),	// h pixel pan

	CMD(EXT_CMD,0x10,0x10),	// ext start address
	CMD(EXT_CMD,0x11,0x00),	// crt overflow
	CMD(EXT_CMD,0x12,0x00),	// enable irq(1)
	CMD(EXT_CMD,0x13,0x00),
	CMD(EXT_CMD,0x14,0xF1),	// vga mem fetches per row
	CMD(EXT_CMD,0x15,0x00),
//	CMD(EXT_CMD,0x30,0x21),	// zero wait(4), PCI burst(20), linear size (3)
	CMD(EXT_CMD,0x31,0x00),	// ext seg write
	CMD(EXT_CMD,0x32,0x00),	// ext seg read
	CMD(EXT_CMD,0x33,0x01),	// linear enable(1), segment on(10), 
	CMD(EXT_CMD,0x50,0x00),	// sprite control
	CMD(EXT_CMD,0x51,0x00),
	CMD(EXT_CMD,0x52,0x00),
	CMD(EXT_CMD,0x53,0x00),
	CMD(EXT_CMD,0x54,0x00),
	CMD(EXT_CMD,0x55,0x00),
	CMD(EXT_CMD,0x56,0x00),	// sprite cursor, DAC access (4)
	CMD(EXT_CMD,0x57,0x01),	// ext graphic mode(1)
	CMD(EXT_CMD,0x58,0x00),	// RAMDAC
	CMD(EXT_CMD,0x59,0x00),
	CMD(EXT_CMD,0x5a,0x00),
//	CMD(EXT_CMD,0x70,0x0B),	// slow mem(1), ras control(3e), 
//	CMD(EXT_CMD,0x71,0x10),	// mem arch(3), sync mem(4), vram(8), mem-type(e0)=60
//	CMD(EXT_CMD,0x72,0x45),	// mem size(3), bus wid(4), refresh(c0)
	CMD(EXT_CMD,0x73,0x30),	// fifo over(81), fifosize(8)
	CMD(EXT_CMD,0x74,0x1b),	// fifo size
	CMD(EXT_CMD,0x75,0x1e),	// fifo size
	CMD(EXT_CMD,0x76,0x00),	// eeprom 
	CMD(EXT_CMD,0x77,0x04),	// color depth(f): 256=1, 32k=6, 64k=2, 16M=4, 4G=3, 256l=9
	CMD(EXT_CMD,0x78,0x00),	
	CMD(EXT_CMD,0x79,0x00),
	CMD(EXT_CMD,0x7a,0xc8),	// cas control
	CMD(EXT_CMD,0xb0,0xD2),	// vclock numerator
	CMD(EXT_CMD,0xb1,0xCE),	// vclock denom
	CMD(EXT_CMD,0xb2,0xDB),	// memclock numerator
	CMD(EXT_CMD,0xb3,0x54),	// memclock denom
	CMD(EXT_CMD,0x90,0x01),	// graphics mode
	CMD(EXT_CMD,0xb9,0x80),	// toggle the clock pll params
	CMD(EXT_CMD,0xb9,0x00),

	CMD(PAL_CMD,0x00,0x14),	// 10=bypass the DAC

};





//---------------------------------------------------------------------------


const int a800x600x8[] = {
	CMD(MISC_CMD,0x00,0xef), // sync polarity(c0), clock select(0c), ram-enable(2), color(1)

	CMD(SEQ_CMD,0x00,0x01),	// sync & async reset: 0 to reset
	CMD(SEQ_CMD,0x01,0x01),	// clock mode, screen-off(20)
	CMD(SEQ_CMD,0x02,0x0f), // write mask
	CMD(SEQ_CMD,0x03,0x00), // font select
	CMD(SEQ_CMD,0x04,0x0e), // ext mem(2), chain4(8)
	CMD(SEQ_CMD,0x00,0x03),	// sync & async reset: 0 to reset

	CMD(CRT_CMD,0x11,0x0b),	// unlock CR0-7
	CMD(CRT_CMD,0,0x7F),	// h total
	CMD(CRT_CMD,1,0x63),	// h disp end
	CMD(CRT_CMD,2,0x64),	// h blank start
	CMD(CRT_CMD,3,0x00),	// h blank end
	CMD(CRT_CMD,4,0x66),	// h retrace start
	CMD(CRT_CMD,5,0x10),	// h retrace end
	CMD(CRT_CMD,6,0x6F),	// v total
	CMD(CRT_CMD,7,0xf0),	// extra bits
	CMD(CRT_CMD,8,0x00),	// row scan
	CMD(CRT_CMD,9,0x60),	// max scan line
	CMD(CRT_CMD,0x0A,0),	// cursor start
	CMD(CRT_CMD,0x0B,0),	// cursor end
	CMD(CRT_CMD,0x0C,0),	// start address (hi)
	CMD(CRT_CMD,0x0D,0),	// start address (lo)
	CMD(CRT_CMD,0x0E,0),	// cursor address
	CMD(CRT_CMD,0x0F,0),	// cursor address
	CMD(CRT_CMD,0x10,0x5B),	// v retrace start
	CMD(CRT_CMD,0x11,0x8F),	// v retrace end
	CMD(CRT_CMD,0x12,0x57),	// v disp enable end
	CMD(CRT_CMD,0x13,0x64),	// offset
	CMD(CRT_CMD,0x14,0x00),	// underline
	CMD(CRT_CMD,0x15,0x59),	// v blank start
	CMD(CRT_CMD,0x16,0x6E),	// v blank end
	CMD(CRT_CMD,0x17,0xE3),	// mode control
	CMD(CRT_CMD,0x18,0xFF),	// line compare

	CMD(GRF_CMD,0x00,0x00),	// set/reset
	CMD(GRF_CMD,0x01,0x00),	// enable set/reset
	CMD(GRF_CMD,0x02,0x00),	// color compare
	CMD(GRF_CMD,0x03,0x00),	// data rotate
	CMD(GRF_CMD,0x04,0x00),	// read map select
	CMD(GRF_CMD,0x05,0x60),	// write mode(3), even/odd(10), srmode(20), mode256(40)
	CMD(GRF_CMD,0x06,0x05),	// graphics(1), chain(2), mmapaddress(c)
	CMD(GRF_CMD,0x07,0x0f),	// color don't care
	CMD(GRF_CMD,0x08,0xff),	// bit mask

	CMD(ATR_CMD,0x00,0x00),	// default palette
	CMD(ATR_CMD,0x01,0x01),
	CMD(ATR_CMD,0x02,0x02),
	CMD(ATR_CMD,0x03,0x03),
	CMD(ATR_CMD,0x04,0x04),
	CMD(ATR_CMD,0x05,0x05),
	CMD(ATR_CMD,0x06,0x06),
	CMD(ATR_CMD,0x07,0x07),
	CMD(ATR_CMD,0x08,0x08),
	CMD(ATR_CMD,0x09,0x09),
	CMD(ATR_CMD,0x0a,0x0a),
	CMD(ATR_CMD,0x0b,0x0b),
	CMD(ATR_CMD,0x0c,0x0c),
	CMD(ATR_CMD,0x0d,0x0d),
	CMD(ATR_CMD,0x0e,0x0e),
	CMD(ATR_CMD,0x0f,0x0f),
	CMD(ATR_CMD,0x10,0x01),	// graphics(1), mono-attr(2), blink(8)
	CMD(ATR_CMD,0x11,0x00),	// overscan color
	CMD(ATR_CMD,0x12,0x0f),	// plane enable
	CMD(ATR_CMD,0x13,0x00),	// h pixel pan

	CMD(EXT_CMD,0x10,0x10),	// ext start address
	CMD(EXT_CMD,0x11,0x00),	// crt overflow
	CMD(EXT_CMD,0x12,0x00),	// enable irq(1)
	CMD(EXT_CMD,0x13,0x00),
	CMD(EXT_CMD,0x14,0x65),	// vga mem fetches per row
	CMD(EXT_CMD,0x15,0x00),
//	CMD(EXT_CMD,0x30,0x21),	// zero wait(4), PCI burst(20), linear size (3)
	CMD(EXT_CMD,0x31,0x00),	// ext seg write
	CMD(EXT_CMD,0x32,0x00),	// ext seg read
	CMD(EXT_CMD,0x33,0x01),	// linear enable(1), segment on(10), 
	CMD(EXT_CMD,0x50,0x00),	// sprite control
	CMD(EXT_CMD,0x51,0x00),
	CMD(EXT_CMD,0x52,0x00),
	CMD(EXT_CMD,0x53,0x00),
	CMD(EXT_CMD,0x54,0x00),
	CMD(EXT_CMD,0x55,0x00),
	CMD(EXT_CMD,0x56,0x00),	// sprite cursor, DAC access (4)
	CMD(EXT_CMD,0x57,0x01),	// ext graphic mode(1)
	CMD(EXT_CMD,0x58,0x00),	// RAMDAC
	CMD(EXT_CMD,0x59,0x00),
	CMD(EXT_CMD,0x5a,0x00),
//	CMD(EXT_CMD,0x70,0x0B),	// slow mem(1), ras control(3e), 
//	CMD(EXT_CMD,0x71,0x10),	// mem arch(3), sync mem(4), vram(8), mem-type(e0)=60
//	CMD(EXT_CMD,0x72,0x45),	// mem size(3), bus wid(4), refresh(c0)
	CMD(EXT_CMD,0x73,0x30),	// fifo over(81), fifosize(8)
	CMD(EXT_CMD,0x74,0x1b),	// fifo size
	CMD(EXT_CMD,0x75,0x1e),	// fifo size
	CMD(EXT_CMD,0x76,0x00),	// eeprom 
	CMD(EXT_CMD,0x77,0x01),	// color depth(f): 256=1, 32k=6, 64k=2, 16M=4, 4G=3, 256l=9
	CMD(EXT_CMD,0x7a,0xc8),	// cas control
	CMD(EXT_CMD,0xb0,0x52),	// vclock numerator
	CMD(EXT_CMD,0xb1,0x85),	// vclock denom
	CMD(EXT_CMD,0xb2,0xDB),	// memclock numerator
	CMD(EXT_CMD,0xb3,0x54),	// memclock denom
	CMD(EXT_CMD,0x90,0x01),	// graphics mode
	CMD(EXT_CMD,0xb9,0x80),	// toggle the clock pll params
	CMD(EXT_CMD,0xb9,0x00),

	CMD(PAL_CMD,0x00,0x14),	// 10=bypass the DAC

};

const int a800x600x16[] = {
	CMD(MISC_CMD,0x00,0xef), // sync polarity(c0), clock select(0c), ram-enable(2), color(1)

	CMD(SEQ_CMD,0x00,0x01),	// sync & async reset: 0 to reset
	CMD(SEQ_CMD,0x01,0x01),	// clock mode, screen-off(20)
	CMD(SEQ_CMD,0x02,0x0f), // write mask
	CMD(SEQ_CMD,0x03,0x00), // font select
	CMD(SEQ_CMD,0x04,0x0e), // ext mem(2), chain4(8)
	CMD(SEQ_CMD,0x00,0x03),	// sync & async reset: 0 to reset

	CMD(CRT_CMD,0x11,0x0b),	// unlock CR0-7
	CMD(CRT_CMD,0,0x7F),	// h total
	CMD(CRT_CMD,1,0x63),	// h disp end
	CMD(CRT_CMD,2,0x64),	// h blank start
	CMD(CRT_CMD,3,0x00),	// h blank end
	CMD(CRT_CMD,4,0x66),	// h retrace start
	CMD(CRT_CMD,5,0x10),	// h retrace end
	CMD(CRT_CMD,6,0x6F),	// v total
	CMD(CRT_CMD,7,0xf0),	// extra bits
	CMD(CRT_CMD,8,0x00),	// row scan
	CMD(CRT_CMD,9,0x60),	// max scan line
	CMD(CRT_CMD,0x0A,0),	// cursor start
	CMD(CRT_CMD,0x0B,0),	// cursor end
	CMD(CRT_CMD,0x0C,0),	// start address (hi)
	CMD(CRT_CMD,0x0D,0),	// start address (lo)
	CMD(CRT_CMD,0x0E,0),	// cursor address
	CMD(CRT_CMD,0x0F,0),	// cursor address
	CMD(CRT_CMD,0x10,0x5B),	// v retrace start
	CMD(CRT_CMD,0x11,0x8F),	// v retrace end
	CMD(CRT_CMD,0x12,0x57),	// v disp enable end
	CMD(CRT_CMD,0x13,0xC8),	// offset
	CMD(CRT_CMD,0x14,0x00),	// underline
	CMD(CRT_CMD,0x15,0x59),	// v blank start
	CMD(CRT_CMD,0x16,0x6E),	// v blank end
	CMD(CRT_CMD,0x17,0xE3),	// mode control
	CMD(CRT_CMD,0x18,0xFF),	// line compare

	CMD(GRF_CMD,0x00,0x00),	// set/reset
	CMD(GRF_CMD,0x01,0x00),	// enable set/reset
	CMD(GRF_CMD,0x02,0x00),	// color compare
	CMD(GRF_CMD,0x03,0x00),	// data rotate
	CMD(GRF_CMD,0x04,0x00),	// read map select
	CMD(GRF_CMD,0x05,0x60),	// write mode(3), even/odd(10), srmode(20), mode256(40)
	CMD(GRF_CMD,0x06,0x05),	// graphics(1), chain(2), mmapaddress(c)
	CMD(GRF_CMD,0x07,0x0f),	// color don't care
	CMD(GRF_CMD,0x08,0xff),	// bit mask

	CMD(ATR_CMD,0x00,0x00),	// default palette
	CMD(ATR_CMD,0x01,0x01),
	CMD(ATR_CMD,0x02,0x02),
	CMD(ATR_CMD,0x03,0x03),
	CMD(ATR_CMD,0x04,0x04),
	CMD(ATR_CMD,0x05,0x05),
	CMD(ATR_CMD,0x06,0x06),
	CMD(ATR_CMD,0x07,0x07),
	CMD(ATR_CMD,0x08,0x08),
	CMD(ATR_CMD,0x09,0x09),
	CMD(ATR_CMD,0x0a,0x0a),
	CMD(ATR_CMD,0x0b,0x0b),
	CMD(ATR_CMD,0x0c,0x0c),
	CMD(ATR_CMD,0x0d,0x0d),
	CMD(ATR_CMD,0x0e,0x0e),
	CMD(ATR_CMD,0x0f,0x0f),
	CMD(ATR_CMD,0x10,0x01),	// graphics(1), mono-attr(2), blink(8)
	CMD(ATR_CMD,0x11,0x00),	// overscan color
	CMD(ATR_CMD,0x12,0x0f),	// plane enable
	CMD(ATR_CMD,0x13,0x00),	// h pixel pan

	CMD(EXT_CMD,0x10,0x10),	// ext start address
	CMD(EXT_CMD,0x11,0x00),	// crt overflow
	CMD(EXT_CMD,0x12,0x00),	// enable irq(1)
	CMD(EXT_CMD,0x13,0x00),
	CMD(EXT_CMD,0x14,0xC9),	// vga mem fetches per row
	CMD(EXT_CMD,0x15,0x00),
//	CMD(EXT_CMD,0x30,0x21),	// zero wait(4), PCI burst(20), linear size (3)
	CMD(EXT_CMD,0x31,0x00),	// ext seg write
	CMD(EXT_CMD,0x32,0x00),	// ext seg read
	CMD(EXT_CMD,0x33,0x01),	// linear enable(1), segment on(10), 
	CMD(EXT_CMD,0x50,0x00),	// sprite control
	CMD(EXT_CMD,0x51,0x00),
	CMD(EXT_CMD,0x52,0x00),
	CMD(EXT_CMD,0x53,0x00),
	CMD(EXT_CMD,0x54,0x00),
	CMD(EXT_CMD,0x55,0x00),
	CMD(EXT_CMD,0x56,0x00),	// sprite cursor, DAC access (4)
	CMD(EXT_CMD,0x57,0x01),	// ext graphic mode(1)
	CMD(EXT_CMD,0x58,0x00),	// RAMDAC
	CMD(EXT_CMD,0x59,0x00),
	CMD(EXT_CMD,0x5a,0x00),
//	CMD(EXT_CMD,0x70,0x0B),	// slow mem(1), ras control(3e), 
//	CMD(EXT_CMD,0x71,0x10),	// mem arch(3), sync mem(4), vram(8), mem-type(e0)=60
//	CMD(EXT_CMD,0x72,0x45),	// mem size(3), bus wid(4), refresh(c0)
	CMD(EXT_CMD,0x73,0x30),	// fifo over(81), fifosize(8)
	CMD(EXT_CMD,0x74,0x1b),	// fifo size
	CMD(EXT_CMD,0x75,0x1e),	// fifo size
	CMD(EXT_CMD,0x76,0x00),	// eeprom 
	CMD(EXT_CMD,0x77,0x02),	// color depth(f): 256=1, 32k=6, 64k=2, 16M=4, 4G=3, 256l=9
	CMD(EXT_CMD,0x7a,0xc8),	// cas control
	CMD(EXT_CMD,0xb0,0x52),	// vclock numerator
	CMD(EXT_CMD,0xb1,0x85),	// vclock denom
	CMD(EXT_CMD,0xb2,0xDB),	// memclock numerator
	CMD(EXT_CMD,0xb3,0x54),	// memclock denom
	CMD(EXT_CMD,0x90,0x01),	// graphics mode
	CMD(EXT_CMD,0xb9,0x80),	// toggle the clock pll params
	CMD(EXT_CMD,0xb9,0x00),

	CMD(PAL_CMD,0x00,0x14),	// 10=bypass the DAC

};

const int a800x600x24[] = {
	CMD(MISC_CMD,0x00,0xef), // sync polarity(c0), clock select(0c), ram-enable(2), color(1)

	CMD(SEQ_CMD,0x00,0x01),	// sync & async reset: 0 to reset
	CMD(SEQ_CMD,0x01,0x01),	// clock mode, screen-off(20)
	CMD(SEQ_CMD,0x02,0x0f), // write mask
	CMD(SEQ_CMD,0x03,0x00), // font select
	CMD(SEQ_CMD,0x04,0x0e), // ext mem(2), chain4(8)
	CMD(SEQ_CMD,0x00,0x03),	// sync & async reset: 0 to reset

	CMD(CRT_CMD,0x11,0x0b),	// unlock CR0-7
	CMD(CRT_CMD,0,0x7F),	// h total
	CMD(CRT_CMD,1,0x63),	// h disp end
	CMD(CRT_CMD,2,0x64),	// h blank start
	CMD(CRT_CMD,3,0x00),	// h blank end
	CMD(CRT_CMD,4,0x66),	// h retrace start
	CMD(CRT_CMD,5,0x10),	// h retrace end
	CMD(CRT_CMD,6,0x6F),	// v total
	CMD(CRT_CMD,7,0xf0),	// extra bits
	CMD(CRT_CMD,8,0x00),	// row scan
	CMD(CRT_CMD,9,0x60),	// max scan line
	CMD(CRT_CMD,0x0A,0),	// cursor start
	CMD(CRT_CMD,0x0B,0),	// cursor end
	CMD(CRT_CMD,0x0C,0),	// start address (hi)
	CMD(CRT_CMD,0x0D,0),	// start address (lo)
	CMD(CRT_CMD,0x0E,0),	// cursor address
	CMD(CRT_CMD,0x0F,0),	// cursor address
	CMD(CRT_CMD,0x10,0x5B),	// v retrace start
	CMD(CRT_CMD,0x11,0x8F),	// v retrace end
	CMD(CRT_CMD,0x12,0x57),	// v disp enable end
	CMD(CRT_CMD,0x13,0x2C),	// offset
	CMD(CRT_CMD,0x14,0x00),	// underline
	CMD(CRT_CMD,0x15,0x59),	// v blank start
	CMD(CRT_CMD,0x16,0x6E),	// v blank end
	CMD(CRT_CMD,0x17,0xE3),	// mode control
	CMD(CRT_CMD,0x18,0xFF),	// line compare

	CMD(GRF_CMD,0x00,0x00),	// set/reset
	CMD(GRF_CMD,0x01,0x00),	// enable set/reset
	CMD(GRF_CMD,0x02,0x00),	// color compare
	CMD(GRF_CMD,0x03,0x00),	// data rotate
	CMD(GRF_CMD,0x04,0x00),	// read map select
	CMD(GRF_CMD,0x05,0x60),	// write mode(3), even/odd(10), srmode(20), mode256(40)
	CMD(GRF_CMD,0x06,0x05),	// graphics(1), chain(2), mmapaddress(c)
	CMD(GRF_CMD,0x07,0x0f),	// color don't care
	CMD(GRF_CMD,0x08,0xff),	// bit mask

	CMD(ATR_CMD,0x00,0x00),	// default palette
	CMD(ATR_CMD,0x01,0x01),
	CMD(ATR_CMD,0x02,0x02),
	CMD(ATR_CMD,0x03,0x03),
	CMD(ATR_CMD,0x04,0x04),
	CMD(ATR_CMD,0x05,0x05),
	CMD(ATR_CMD,0x06,0x06),
	CMD(ATR_CMD,0x07,0x07),
	CMD(ATR_CMD,0x08,0x08),
	CMD(ATR_CMD,0x09,0x09),
	CMD(ATR_CMD,0x0a,0x0a),
	CMD(ATR_CMD,0x0b,0x0b),
	CMD(ATR_CMD,0x0c,0x0c),
	CMD(ATR_CMD,0x0d,0x0d),
	CMD(ATR_CMD,0x0e,0x0e),
	CMD(ATR_CMD,0x0f,0x0f),
	CMD(ATR_CMD,0x10,0x01),	// graphics(1), mono-attr(2), blink(8)
	CMD(ATR_CMD,0x11,0x00),	// overscan color
	CMD(ATR_CMD,0x12,0x0f),	// plane enable
	CMD(ATR_CMD,0x13,0x00),	// h pixel pan

	CMD(EXT_CMD,0x10,0x10),	// ext start address
	CMD(EXT_CMD,0x11,0x00),	// crt overflow
	CMD(EXT_CMD,0x12,0x00),	// enable irq(1)
	CMD(EXT_CMD,0x13,0x00),
	CMD(EXT_CMD,0x14,0x2D),	// vga mem fetches per row
	CMD(EXT_CMD,0x15,0x11),
//	CMD(EXT_CMD,0x30,0x21),	// zero wait(4), PCI burst(20), linear size (3)
	CMD(EXT_CMD,0x31,0x00),	// ext seg write
	CMD(EXT_CMD,0x32,0x00),	// ext seg read
	CMD(EXT_CMD,0x33,0x01),	// linear enable(1), segment on(10), 
	CMD(EXT_CMD,0x50,0x00),	// sprite control
	CMD(EXT_CMD,0x51,0x00),
	CMD(EXT_CMD,0x52,0x00),
	CMD(EXT_CMD,0x53,0x00),
	CMD(EXT_CMD,0x54,0x00),
	CMD(EXT_CMD,0x55,0x00),
	CMD(EXT_CMD,0x56,0x00),	// sprite cursor, DAC access (4)
	CMD(EXT_CMD,0x57,0x01),	// ext graphic mode(1)
	CMD(EXT_CMD,0x58,0x00),	// RAMDAC
	CMD(EXT_CMD,0x59,0x00),
	CMD(EXT_CMD,0x5a,0x00),
//	CMD(EXT_CMD,0x70,0x0B),	// slow mem(1), ras control(3e), 
//	CMD(EXT_CMD,0x71,0x10),	// mem arch(3), sync mem(4), vram(8), mem-type(e0)=60
//	CMD(EXT_CMD,0x72,0x45),	// mem size(3), bus wid(4), refresh(c0)
	CMD(EXT_CMD,0x73,0x30),	// fifo over(81), fifosize(8)
	CMD(EXT_CMD,0x74,0x1b),	// fifo size
	CMD(EXT_CMD,0x75,0x1e),	// fifo size
	CMD(EXT_CMD,0x76,0x00),	// eeprom 
	CMD(EXT_CMD,0x77,0x04),	// color depth(f): 256=1, 32k=6, 64k=2, 16M=4, 4G=3, 256l=9
	CMD(EXT_CMD,0x7a,0xc8),	// cas control
	CMD(EXT_CMD,0xb0,0x52),	// vclock numerator
	CMD(EXT_CMD,0xb1,0x85),	// vclock denom
	CMD(EXT_CMD,0xb2,0xDB),	// memclock numerator
	CMD(EXT_CMD,0xb3,0x54),	// memclock denom
	CMD(EXT_CMD,0x90,0x01),	// graphics mode
	CMD(EXT_CMD,0xb9,0x80),	// toggle the clock pll params
	CMD(EXT_CMD,0xb9,0x00),

	CMD(PAL_CMD,0x00,0x14),	// 10=bypass the DAC

};

//---------------------------------------------------------------------------

const int a1024x768x8[] = {
	CMD(MISC_CMD,0x00,0xef), // sync polarity(c0), clock select(0c), ram-enable(2), color(1)

	CMD(SEQ_CMD,0x00,0x01),	// sync & async reset: 0 to reset
	CMD(SEQ_CMD,0x01,0x01),	// clock mode, screen-off(20)
	CMD(SEQ_CMD,0x02,0x0f), // write mask
	CMD(SEQ_CMD,0x03,0x00), // font select
	CMD(SEQ_CMD,0x04,0x0e), // ext mem(2), chain4(8)
	CMD(SEQ_CMD,0x00,0x03),	// sync & async reset: 0 to reset

	CMD(CRT_CMD,0x11,0x0b),	// unlock CR0-7
	CMD(CRT_CMD,0,0x9F),	// h total
	CMD(CRT_CMD,1,0x7F),	// h disp end
	CMD(CRT_CMD,2,0x80),	// h blank start
	CMD(CRT_CMD,3,0x80),	// h blank end
	CMD(CRT_CMD,4,0x8B),	// h retrace start
	CMD(CRT_CMD,5,0x94),	// h retrace end
	CMD(CRT_CMD,6,0x1E),	// v total
	CMD(CRT_CMD,7,0xFD),	// extra bits
	CMD(CRT_CMD,8,0x00),	// row scan
	CMD(CRT_CMD,9,0x60),	// max scan line
	CMD(CRT_CMD,0x0A,0x0E),	// cursor start	!!!
	CMD(CRT_CMD,0x0B,0x0E),	// cursor end	!!!
	CMD(CRT_CMD,0x0C,0),	// start address (hi)
	CMD(CRT_CMD,0x0D,0),	// start address (lo)
	CMD(CRT_CMD,0x0E,0x04),	// cursor address	!!!
	CMD(CRT_CMD,0x0F,0),	// cursor address
	CMD(CRT_CMD,0x10,0x03),	// v retrace start
	CMD(CRT_CMD,0x11,0x86),	// v retrace end
	CMD(CRT_CMD,0x12,0xFF),	// v disp enable end
	CMD(CRT_CMD,0x13,0x80),	// offset
	CMD(CRT_CMD,0x14,0x0F),	// underline
	CMD(CRT_CMD,0x15,0x00),	// v blank start
	CMD(CRT_CMD,0x16,0x1E),	// v blank end
	CMD(CRT_CMD,0x17,0xE3),	// mode control
	CMD(CRT_CMD,0x18,0xFF),	// line compare

	CMD(GRF_CMD,0x00,0x00),	// set/reset
	CMD(GRF_CMD,0x01,0x00),	// enable set/reset
	CMD(GRF_CMD,0x02,0x00),	// color compare
	CMD(GRF_CMD,0x03,0x00),	// data rotate
	CMD(GRF_CMD,0x04,0x00),	// read map select
	CMD(GRF_CMD,0x05,0x60),	// write mode(3), even/odd(10), srmode(20), mode256(40)
	CMD(GRF_CMD,0x06,0x05),	// graphics(1), chain(2), mmapaddress(c)
	CMD(GRF_CMD,0x07,0x0f),	// color don't care
	CMD(GRF_CMD,0x08,0xff),	// bit mask

	CMD(ATR_CMD,0x00,0x00),	// default palette
	CMD(ATR_CMD,0x01,0x01),
	CMD(ATR_CMD,0x02,0x02),
	CMD(ATR_CMD,0x03,0x03),
	CMD(ATR_CMD,0x04,0x04),
	CMD(ATR_CMD,0x05,0x05),
	CMD(ATR_CMD,0x06,0x06),
	CMD(ATR_CMD,0x07,0x07),
	CMD(ATR_CMD,0x08,0x08),
	CMD(ATR_CMD,0x09,0x09),
	CMD(ATR_CMD,0x0a,0x0a),
	CMD(ATR_CMD,0x0b,0x0b),
	CMD(ATR_CMD,0x0c,0x0c),
	CMD(ATR_CMD,0x0d,0x0d),
	CMD(ATR_CMD,0x0e,0x0e),
	CMD(ATR_CMD,0x0f,0x0f),
	CMD(ATR_CMD,0x10,0x01),	// graphics(1), mono-attr(2), blink(8)
	CMD(ATR_CMD,0x11,0x00),	// overscan color
	CMD(ATR_CMD,0x12,0x0f),	// plane enable
	CMD(ATR_CMD,0x13,0x00),	// h pixel pan

	CMD(EXT_CMD,0x10,0x10),	// ext start address
	CMD(EXT_CMD,0x11,0x00),	// crt overflow
	CMD(EXT_CMD,0x12,0x00),	// enable irq(1)
	CMD(EXT_CMD,0x13,0x00),
	CMD(EXT_CMD,0x14,0x81),	// vga mem fetches per row
	CMD(EXT_CMD,0x15,0x00),
//	CMD(EXT_CMD,0x30,0x21),	// zero wait(4), PCI burst(20), linear size (3)
	CMD(EXT_CMD,0x31,0x00),	// ext seg write
	CMD(EXT_CMD,0x32,0x00),	// ext seg read
	CMD(EXT_CMD,0x33,0x01),	// linear enable(1), segment on(10), 

	CMD(EXT_CMD,0x50,0x00),	// sprite control
	CMD(EXT_CMD,0x51,0x00),
	CMD(EXT_CMD,0x52,0x00),
	CMD(EXT_CMD,0x53,0x00),
	CMD(EXT_CMD,0x54,0x00),
	CMD(EXT_CMD,0x55,0x00),
	CMD(EXT_CMD,0x56,0x00),	// sprite cursor, DAC access (4)
	CMD(EXT_CMD,0x57,0x01),	// ext graphic mode(1)
	CMD(EXT_CMD,0x58,0x00),	// RAMDAC
	CMD(EXT_CMD,0x59,0x00),
	CMD(EXT_CMD,0x5a,0x00),

//	CMD(EXT_CMD,0x70,0x0B),	// slow mem(1), ras control(3e), 
//	CMD(EXT_CMD,0x71,0x10), // mem arch(3), sync mem(4), vram(8), mem-type(e0)=60
//	CMD(EXT_CMD,0x72,0x45),	// mem size(3), bus wid(4), refresh(c0)
	CMD(EXT_CMD,0x73,0x30),	// fifo over(81), fifosize(8)
	CMD(EXT_CMD,0x74,0x1b),	// fifo size
	CMD(EXT_CMD,0x75,0x1e),	// fifo size
	CMD(EXT_CMD,0x76,0x00),	// eeprom 
	CMD(EXT_CMD,0x77,0x01),	// color depth(f): 256=1, 32k=6, 64k=2, 16M=4, 4G=3, 256l=9
	CMD(EXT_CMD,0x7a,0xc8),	// cas control

	CMD(EXT_CMD,0xb0,0xD0),	// vclock numerator
	CMD(EXT_CMD,0xb1,0x52),	// vclock denom
	CMD(EXT_CMD,0xb2,0xDB),	// memclock numerator
	CMD(EXT_CMD,0xb3,0x54),	// memclock denom

	CMD(EXT_CMD,0x90,0x01),	// graphics mode

	CMD(EXT_CMD,0xb9,0x80),	// toggle the clock pll params
	CMD(EXT_CMD,0xb9,0x00),

	CMD(PAL_CMD,0x00,0x14),	// 10=bypass the DAC

};


const int a1024x768x16[] = {
	CMD(MISC_CMD,0x00,0xef), // sync polarity(c0), clock select(0c), ram-enable(2), color(1)

	CMD(SEQ_CMD,0x00,0x01),	// sync & async reset: 0 to reset
	CMD(SEQ_CMD,0x01,0x01),	// clock mode, screen-off(20)
	CMD(SEQ_CMD,0x02,0x0f), // write mask
	CMD(SEQ_CMD,0x03,0x00), // font select
	CMD(SEQ_CMD,0x04,0x0e), // ext mem(2), chain4(8)
	CMD(SEQ_CMD,0x00,0x03),	// sync & async reset: 0 to reset

	CMD(CRT_CMD,0x11,0x0b),	// unlock CR0-7
	CMD(CRT_CMD,0,0x9F),	// h total
	CMD(CRT_CMD,1,0x7F),	// h disp end
	CMD(CRT_CMD,2,0x80),	// h blank start
	CMD(CRT_CMD,3,0x80),	// h blank end
	CMD(CRT_CMD,4,0x8B),	// h retrace start
	CMD(CRT_CMD,5,0x94),	// h retrace end
	CMD(CRT_CMD,6,0x1E),	// v total
	CMD(CRT_CMD,7,0xFD),	// extra bits
	CMD(CRT_CMD,8,0x00),	// row scan
	CMD(CRT_CMD,9,0x60),	// max scan line
	CMD(CRT_CMD,0x0A,0x0E),	// cursor start	!!!
	CMD(CRT_CMD,0x0B,0x0E),	// cursor end	!!!
	CMD(CRT_CMD,0x0C,0),	// start address (hi)
	CMD(CRT_CMD,0x0D,0),	// start address (lo)
	CMD(CRT_CMD,0x0E,0x04),	// cursor address	!!!
	CMD(CRT_CMD,0x0F,0),	// cursor address
	CMD(CRT_CMD,0x10,0x03),	// v retrace start
	CMD(CRT_CMD,0x11,0x86),	// v retrace end
	CMD(CRT_CMD,0x12,0xFF),	// v disp enable end
	CMD(CRT_CMD,0x13,0x00),	// offset
	CMD(CRT_CMD,0x14,0x0F),	// underline
	CMD(CRT_CMD,0x15,0x00),	// v blank start
	CMD(CRT_CMD,0x16,0x1E),	// v blank end
	CMD(CRT_CMD,0x17,0xE3),	// mode control
	CMD(CRT_CMD,0x18,0xFF),	// line compare

	CMD(GRF_CMD,0x00,0x00),	// set/reset
	CMD(GRF_CMD,0x01,0x00),	// enable set/reset
	CMD(GRF_CMD,0x02,0x00),	// color compare
	CMD(GRF_CMD,0x03,0x00),	// data rotate
	CMD(GRF_CMD,0x04,0x00),	// read map select
	CMD(GRF_CMD,0x05,0x60),	// write mode(3), even/odd(10), srmode(20), mode256(40)
	CMD(GRF_CMD,0x06,0x05),	// graphics(1), chain(2), mmapaddress(c)
	CMD(GRF_CMD,0x07,0x0f),	// color don't care
	CMD(GRF_CMD,0x08,0xff),	// bit mask

	CMD(ATR_CMD,0x00,0x00),	// default palette
	CMD(ATR_CMD,0x01,0x01),
	CMD(ATR_CMD,0x02,0x02),
	CMD(ATR_CMD,0x03,0x03),
	CMD(ATR_CMD,0x04,0x04),
	CMD(ATR_CMD,0x05,0x05),
	CMD(ATR_CMD,0x06,0x06),
	CMD(ATR_CMD,0x07,0x07),
	CMD(ATR_CMD,0x08,0x08),
	CMD(ATR_CMD,0x09,0x09),
	CMD(ATR_CMD,0x0a,0x0a),
	CMD(ATR_CMD,0x0b,0x0b),
	CMD(ATR_CMD,0x0c,0x0c),
	CMD(ATR_CMD,0x0d,0x0d),
	CMD(ATR_CMD,0x0e,0x0e),
	CMD(ATR_CMD,0x0f,0x0f),
	CMD(ATR_CMD,0x10,0x01),	// graphics(1), mono-attr(2), blink(8)
	CMD(ATR_CMD,0x11,0x00),	// overscan color
	CMD(ATR_CMD,0x12,0x0f),	// plane enable
	CMD(ATR_CMD,0x13,0x00),	// h pixel pan

	CMD(EXT_CMD,0x10,0x10),	// ext start address
	CMD(EXT_CMD,0x11,0x00),	// crt overflow
	CMD(EXT_CMD,0x12,0x00),	// enable irq(1)
	CMD(EXT_CMD,0x13,0x00),
	CMD(EXT_CMD,0x14,0x01),	// vga mem fetches per row
	CMD(EXT_CMD,0x15,0x11),
//	CMD(EXT_CMD,0x30,0x21),	// zero wait(4), PCI burst(20), linear size (3)
	CMD(EXT_CMD,0x31,0x00),	// ext seg write
	CMD(EXT_CMD,0x32,0x00),	// ext seg read
	CMD(EXT_CMD,0x33,0x01),	// linear enable(1), segment on(10), 

	CMD(EXT_CMD,0x50,0x00),	// sprite control
	CMD(EXT_CMD,0x51,0x00),
	CMD(EXT_CMD,0x52,0x00),
	CMD(EXT_CMD,0x53,0x00),
	CMD(EXT_CMD,0x54,0x00),
	CMD(EXT_CMD,0x55,0x00),
	CMD(EXT_CMD,0x56,0x00),	// sprite cursor, DAC access (4)
	CMD(EXT_CMD,0x57,0x01),	// ext graphic mode(1)
	CMD(EXT_CMD,0x58,0x00),	// RAMDAC
	CMD(EXT_CMD,0x59,0x00),
	CMD(EXT_CMD,0x5a,0x00),

//	CMD(EXT_CMD,0x70,0x0B),	// slow mem(1), ras control(3e), 
//	CMD(EXT_CMD,0x71,0x10), // mem arch(3), sync mem(4), vram(8), mem-type(e0)=60
//	CMD(EXT_CMD,0x72,0x45),	// mem size(3), bus wid(4), refresh(c0)
	CMD(EXT_CMD,0x73,0x30),	// fifo over(81), fifosize(8)
	CMD(EXT_CMD,0x74,0x1b),	// fifo size
	CMD(EXT_CMD,0x75,0x1e),	// fifo size
	CMD(EXT_CMD,0x76,0x00),	// eeprom 
	CMD(EXT_CMD,0x77,0x02),	// color depth(f): 256=1, 32k=6, 64k=2, 16M=4, 4G=3, 256l=9
	CMD(EXT_CMD,0x7a,0xc8),	// cas control

	CMD(EXT_CMD,0xb0,0xD0),	// vclock numerator
	CMD(EXT_CMD,0xb1,0x52),	// vclock denom
	CMD(EXT_CMD,0xb2,0xDB),	// memclock numerator
	CMD(EXT_CMD,0xb3,0x54),	// memclock denom

	CMD(EXT_CMD,0x90,0x01),	// graphics mode

	CMD(EXT_CMD,0xb9,0x80),	// toggle the clock pll params
	CMD(EXT_CMD,0xb9,0x00),

	CMD(PAL_CMD,0x00,0x14),	// 10=bypass the DAC

};


//---------------------------------------------------------------------------
//60 Hz
const int a1280x1024x8[] = {
	CMD(MISC_CMD,0x00,0xef), // sync polarity(c0), clock select(0c), ram-enable(2), color(1)

	CMD(SEQ_CMD,0x00,0x01),	// sync & async reset: 0 to reset
	CMD(SEQ_CMD,0x01,0x01),	// clock mode, screen-off(20)
	CMD(SEQ_CMD,0x02,0x0f), // write mask
	CMD(SEQ_CMD,0x03,0x00), // font select
	CMD(SEQ_CMD,0x04,0x0e), // ext mem(2), chain4(8)
	CMD(SEQ_CMD,0x00,0x03),	// sync & async reset: 0 to reset

	CMD(CRT_CMD,0x11,0x0b),	// unlock CR0-7
	CMD(CRT_CMD,0,0xCE),	// h total
	CMD(CRT_CMD,1,0x9F),	// h disp end
	CMD(CRT_CMD,2,0xA0),	// h blank start
	CMD(CRT_CMD,3,0x8F),	// h blank end
	CMD(CRT_CMD,4,0xA2),	// h retrace start
	CMD(CRT_CMD,5,0x1F),	// h retrace end
	CMD(CRT_CMD,6,0x28),	// v total
	CMD(CRT_CMD,7,0x52),	// extra bits
	CMD(CRT_CMD,8,0x00),	// row scan
	CMD(CRT_CMD,9,0x40),	// max scan line
	CMD(CRT_CMD,0x0A,0x00),	// cursor start	!!!
	CMD(CRT_CMD,0x0B,0x00),	// cursor end	!!!
	CMD(CRT_CMD,0x0C,0),	// start address (hi)
	CMD(CRT_CMD,0x0D,0),	// start address (lo)
	CMD(CRT_CMD,0x0E,0x00),	// cursor address	!!!
	CMD(CRT_CMD,0x0F,0),	// cursor address
	CMD(CRT_CMD,0x10,0x08),	// v retrace start
	CMD(CRT_CMD,0x11,0x8F),	// v retrace end
	CMD(CRT_CMD,0x12,0xFF),	// v disp enable end
	CMD(CRT_CMD,0x13,0xA0),	// offset
	CMD(CRT_CMD,0x14,0x00),	// underline
	CMD(CRT_CMD,0x15,0x03),	// v blank start
	CMD(CRT_CMD,0x16,0x27),	// v blank end
	CMD(CRT_CMD,0x17,0xE3),	// mode control
	CMD(CRT_CMD,0x18,0xFF),	// line compare

	CMD(GRF_CMD,0x00,0x00),	// set/reset
	CMD(GRF_CMD,0x01,0x00),	// enable set/reset
	CMD(GRF_CMD,0x02,0x00),	// color compare
	CMD(GRF_CMD,0x03,0x00),	// data rotate
	CMD(GRF_CMD,0x04,0x00),	// read map select
	CMD(GRF_CMD,0x05,0x60),	// write mode(3), even/odd(10), srmode(20), mode256(40)
	CMD(GRF_CMD,0x06,0x05),	// graphics(1), chain(2), mmapaddress(c)
	CMD(GRF_CMD,0x07,0x0f),	// color don't care
	CMD(GRF_CMD,0x08,0xff),	// bit mask

	CMD(ATR_CMD,0x00,0x00),	// default palette
	CMD(ATR_CMD,0x01,0x01),
	CMD(ATR_CMD,0x02,0x02),
	CMD(ATR_CMD,0x03,0x03),
	CMD(ATR_CMD,0x04,0x04),
	CMD(ATR_CMD,0x05,0x05),
	CMD(ATR_CMD,0x06,0x06),
	CMD(ATR_CMD,0x07,0x07),
	CMD(ATR_CMD,0x08,0x08),
	CMD(ATR_CMD,0x09,0x09),
	CMD(ATR_CMD,0x0a,0x0a),
	CMD(ATR_CMD,0x0b,0x0b),
	CMD(ATR_CMD,0x0c,0x0c),
	CMD(ATR_CMD,0x0d,0x0d),
	CMD(ATR_CMD,0x0e,0x0e),
	CMD(ATR_CMD,0x0f,0x0f),
	CMD(ATR_CMD,0x10,0x01),	// graphics(1), mono-attr(2), blink(8)
	CMD(ATR_CMD,0x11,0x00),	// overscan color
	CMD(ATR_CMD,0x12,0x0f),	// plane enable
	CMD(ATR_CMD,0x13,0x00),	// h pixel pan

	CMD(EXT_CMD,0x10,0x10),	// ext start address
	CMD(EXT_CMD,0x11,0x1D),	// crt overflow
	CMD(EXT_CMD,0x12,0x00),	// enable irq(1)
	CMD(EXT_CMD,0x13,0x00),
	CMD(EXT_CMD,0x14,0xA1),	// vga mem fetches per row
	CMD(EXT_CMD,0x15,0x00),
//	CMD(EXT_CMD,0x30,0x21),	// zero wait(4), PCI burst(20), linear size (3)
	CMD(EXT_CMD,0x31,0x00),	// ext seg write
	CMD(EXT_CMD,0x32,0x00),	// ext seg read
	CMD(EXT_CMD,0x33,0x01),	// linear enable(1), segment on(10), 

	CMD(EXT_CMD,0x50,0x00),	// sprite control
	CMD(EXT_CMD,0x51,0x00),
	CMD(EXT_CMD,0x52,0x00),
	CMD(EXT_CMD,0x53,0x00),
	CMD(EXT_CMD,0x54,0x00),
	CMD(EXT_CMD,0x55,0x00),
	CMD(EXT_CMD,0x56,0x00),	// sprite cursor, DAC access (4)
	CMD(EXT_CMD,0x57,0x01),	// ext graphic mode(1)
	CMD(EXT_CMD,0x58,0x00),	// RAMDAC
	CMD(EXT_CMD,0x59,0x00),
	CMD(EXT_CMD,0x5a,0x00),

//	CMD(EXT_CMD,0x70,0x0B),	// slow mem(1), ras control(3e), 
//	CMD(EXT_CMD,0x71,0x10), // mem arch(3), sync mem(4), vram(8), mem-type(e0)=60
//	CMD(EXT_CMD,0x72,0x45),	// mem size(3), bus wid(4), refresh(c0)
	CMD(EXT_CMD,0x73,0x30),	// fifo over(81), fifosize(8)
	CMD(EXT_CMD,0x74,0x1b),	// fifo size
	CMD(EXT_CMD,0x75,0x1e),	// fifo size
	CMD(EXT_CMD,0x76,0x00),	// eeprom 
	CMD(EXT_CMD,0x77,0x01),	// color depth(f): 256=1, 32k=6, 64k=2, 16M=4, 4G=3, 256l=9
	CMD(EXT_CMD,0x7a,0xc8),	// cas control

	CMD(EXT_CMD,0xb0,0xB4),	// vclock numerator
	CMD(EXT_CMD,0xb1,0x4B),	// vclock denom
	CMD(EXT_CMD,0xb2,0xDB),	// memclock numerator
	CMD(EXT_CMD,0xb3,0x54),	// memclock denom

	CMD(EXT_CMD,0x90,0x01),	// graphics mode

	CMD(EXT_CMD,0xb9,0x80),	// toggle the clock pll params
	CMD(EXT_CMD,0xb9,0x00),

	CMD(PAL_CMD,0x00,0x14),	// 10=bypass the DAC

};


//---------------------------------------------------------------------------
//60 Hz
const int a1600x1200x8[] = {
	CMD(MISC_CMD,0x00,0xef), // sync polarity(c0), clock select(0c), ram-enable(2), color(1)

	CMD(SEQ_CMD,0x00,0x01),	// sync & async reset: 0 to reset
	CMD(SEQ_CMD,0x01,0x01),	// clock mode, screen-off(20)
	CMD(SEQ_CMD,0x02,0x0f), // write mask
	CMD(SEQ_CMD,0x03,0x00), // font select
	CMD(SEQ_CMD,0x04,0x0e), // ext mem(2), chain4(8)
	CMD(SEQ_CMD,0x00,0x03),	// sync & async reset: 0 to reset

	CMD(CRT_CMD,0x11,0x0b),	// unlock CR0-7
	CMD(CRT_CMD,0,0xFF),	// h total
	CMD(CRT_CMD,1,0xC7),	// h disp end
	CMD(CRT_CMD,2,0xC9),	// h blank start
	CMD(CRT_CMD,3,0x9F),	// h blank end
	CMD(CRT_CMD,4,0xCF),	// h retrace start
	CMD(CRT_CMD,5,0xA0),	// h retrace end
	CMD(CRT_CMD,6,0xFE),	// v total
	CMD(CRT_CMD,7,0x10),	// extra bits
	CMD(CRT_CMD,8,0x00),	// row scan
	CMD(CRT_CMD,9,0x40),	// max scan line
	CMD(CRT_CMD,0x0A,0x00),	// cursor start	!!!
	CMD(CRT_CMD,0x0B,0x00),	// cursor end	!!!
	CMD(CRT_CMD,0x0C,0),	// start address (hi)
	CMD(CRT_CMD,0x0D,0),	// start address (lo)
	CMD(CRT_CMD,0x0E,0x00),	// cursor address	!!!
	CMD(CRT_CMD,0x0F,0),	// cursor address
	CMD(CRT_CMD,0x10,0xCF),	// v retrace start
	CMD(CRT_CMD,0x11,0x89),	// v retrace end
	CMD(CRT_CMD,0x12,0xAF),	// v disp enable end
	CMD(CRT_CMD,0x13,0xC8),	// offset
	CMD(CRT_CMD,0x14,0x00),	// underline
	CMD(CRT_CMD,0x15,0xBC),	// v blank start
	CMD(CRT_CMD,0x16,0xF1),	// v blank end
	CMD(CRT_CMD,0x17,0xE3),	// mode control
	CMD(CRT_CMD,0x18,0xFF),	// line compare

	CMD(GRF_CMD,0x00,0x00),	// set/reset
	CMD(GRF_CMD,0x01,0x00),	// enable set/reset
	CMD(GRF_CMD,0x02,0x00),	// color compare
	CMD(GRF_CMD,0x03,0x00),	// data rotate
	CMD(GRF_CMD,0x04,0x00),	// read map select
	CMD(GRF_CMD,0x05,0x60),	// write mode(3), even/odd(10), srmode(20), mode256(40)
	CMD(GRF_CMD,0x06,0x05),	// graphics(1), chain(2), mmapaddress(c)
	CMD(GRF_CMD,0x07,0x0f),	// color don't care
	CMD(GRF_CMD,0x08,0xff),	// bit mask

	CMD(ATR_CMD,0x00,0x00),	// default palette
	CMD(ATR_CMD,0x01,0x01),
	CMD(ATR_CMD,0x02,0x02),
	CMD(ATR_CMD,0x03,0x03),
	CMD(ATR_CMD,0x04,0x04),
	CMD(ATR_CMD,0x05,0x05),
	CMD(ATR_CMD,0x06,0x06),
	CMD(ATR_CMD,0x07,0x07),
	CMD(ATR_CMD,0x08,0x08),
	CMD(ATR_CMD,0x09,0x09),
	CMD(ATR_CMD,0x0a,0x0a),
	CMD(ATR_CMD,0x0b,0x0b),
	CMD(ATR_CMD,0x0c,0x0c),
	CMD(ATR_CMD,0x0d,0x0d),
	CMD(ATR_CMD,0x0e,0x0e),
	CMD(ATR_CMD,0x0f,0x0f),
	CMD(ATR_CMD,0x10,0x01),	// graphics(1), mono-attr(2), blink(8)
	CMD(ATR_CMD,0x11,0x00),	// overscan color
	CMD(ATR_CMD,0x12,0x0f),	// plane enable
	CMD(ATR_CMD,0x13,0x00),	// h pixel pan

	CMD(EXT_CMD,0x10,0x10),	// ext start address
	CMD(EXT_CMD,0x11,0x1F),	// crt overflow
	CMD(EXT_CMD,0x12,0x00),	// enable irq(1)
	CMD(EXT_CMD,0x13,0x00),
	CMD(EXT_CMD,0x14,0xC9),	// vga mem fetches per row
	CMD(EXT_CMD,0x15,0x00),
//	CMD(EXT_CMD,0x30,0x21),	// zero wait(4), PCI burst(20), linear size (3)
	CMD(EXT_CMD,0x31,0x00),	// ext seg write
	CMD(EXT_CMD,0x32,0x00),	// ext seg read
	CMD(EXT_CMD,0x33,0x01),	// linear enable(1), segment on(10), 

	CMD(EXT_CMD,0x50,0x00),	// sprite control
	CMD(EXT_CMD,0x51,0x00),
	CMD(EXT_CMD,0x52,0x00),
	CMD(EXT_CMD,0x53,0x00),
	CMD(EXT_CMD,0x54,0x00),
	CMD(EXT_CMD,0x55,0x00),
	CMD(EXT_CMD,0x56,0x00),	// sprite cursor, DAC access (4)
	CMD(EXT_CMD,0x57,0x01),	// ext graphic mode(1)
	CMD(EXT_CMD,0x58,0x00),	// RAMDAC
	CMD(EXT_CMD,0x59,0x00),
	CMD(EXT_CMD,0x5a,0x00),

//	CMD(EXT_CMD,0x70,0x0B),	// slow mem(1), ras control(3e), 
//	CMD(EXT_CMD,0x71,0x10), // mem arch(3), sync mem(4), vram(8), mem-type(e0)=60
//	CMD(EXT_CMD,0x72,0x45),	// mem size(3), bus wid(4), refresh(c0)
	CMD(EXT_CMD,0x73,0x30),	// fifo over(81), fifosize(8)
	CMD(EXT_CMD,0x74,0x1b),	// fifo size
	CMD(EXT_CMD,0x75,0x1e),	// fifo size
	CMD(EXT_CMD,0x76,0x00),	// eeprom 
	CMD(EXT_CMD,0x77,0x01),	// color depth(f): 256=1, 32k=6, 64k=2, 16M=4, 4G=3, 256l=9
	CMD(EXT_CMD,0x7a,0xc8),	// cas control

	CMD(EXT_CMD,0xb0,0xBD),	// vclock numerator
	CMD(EXT_CMD,0xb1,0x10),	// vclock denom
	CMD(EXT_CMD,0xb2,0xDB),	// memclock numerator
	CMD(EXT_CMD,0xb3,0x54),	// memclock denom

	CMD(EXT_CMD,0x90,0x01),	// graphics mode

	CMD(EXT_CMD,0xb9,0x80),	// toggle the clock pll params
	CMD(EXT_CMD,0xb9,0x00),

	CMD(PAL_CMD,0x00,0x14),	// 10=bypass the DAC

};






void igs_init(void) {
	pVidPort = (volatile char*)0xd0800000;
}


// igs_setregs
//	parse the command array and 
//	send data to the chip in various ways
void igs_setregs(const int* regs, int n) {
	int i;
	volatile char c;

	for (i=0; i<n; i++) {
		// pull out the command part
		switch(regs[i]>>16) {
			case CRT_CMD:
				pVidPort[0x3d4] = regs[i]>>8;
				udelay(10);
				pVidPort[0x3d5] = regs[i];
				break;
			case ATR_CMD:
				c = pVidPort[0x3da];
				udelay(10);
				pVidPort[0x3c0] = regs[i]>>8;
				udelay(10);
				pVidPort[0x3c0] = regs[i];
				break;
			case SEQ_CMD:
				pVidPort[0x3c4] = regs[i]>>8;
				udelay(10);
				pVidPort[0x3c5] = regs[i];
				break;
			case GRF_CMD:
			case EXT_CMD:
				pVidPort[0x3ce] = regs[i]>>8;
				udelay(10);
				pVidPort[0x3cf] = regs[i];
				break;
			case MISC_CMD:
				pVidPort[0x3c2] = regs[i];
				break;
			case PAL_CMD:
				pVidPort[0x3ce] = 0x56;
				c = pVidPort[0x3cf];
				pVidPort[0x3cf] = c | 4;
				pVidPort[0x3c6] = regs[i];
				pVidPort[0x3cf] = c;
				break;
		}
	}
}

int igs_getmode(void) {
	return mode;
}

int igs_setmode(int new_mode) {
	int old_mode = mode;

//printk("igs_setmode: going from mode %x to mode %x\n",old_mode, new_mode);
	if (new_mode == old_mode)
		return mode;

	if (!pVidPort)
		igs_init();

	if (new_mode == IGS_TEXT)
	{
	/* patb 14Sept98
	 * assume that the caller will restore some kind of text mode
	 * and font
	 * we know that svgalib and X both do that
	 */
	//	igs_setregs(aTEXT,sizeof(aTEXT)/sizeof(aTEXT[0]));
	//	SET_CRT(0xC,nStartAddr);
	//	SET_CRT(0xD,(nStartAddr>>8));
	//	SET_CRT(0xE,nCursorAddr);
	//	SET_CRT(0xF,(nCursorAddr>>8));
	//	con_set_font(0,0);
	}
	else
	{
		// save start addr
		nStartAddr = GET_CRT(0xC) + (GET_CRT(0xD) << 8);
		nCursorAddr = GET_CRT(0xE) + (GET_CRT(0xF) << 8);

		switch(new_mode)
		{
		case IGS_640x480x1:
			igs_setregs(a640x480x1,sizeof(a640x480x1)/sizeof(a640x480x1[0]));
			break;
		case IGS_640x480x8:
			igs_setregs(a640x480x8,sizeof(a640x480x8)/sizeof(a640x480x8[0]));
			break;
		case IGS_640x480x16:
			igs_setregs(a640x480x16,sizeof(a640x480x16)/sizeof(a640x480x16[0]));
			break;
		case IGS_640x480x24:
			igs_setregs(a640x480x24,sizeof(a640x480x24)/sizeof(a640x480x24[0]));
			break;
		case IGS_800x600x8:
			igs_setregs(a800x600x8,sizeof(a800x600x8)/sizeof(a800x600x8[0]));
			break;
		case IGS_800x600x16:
			igs_setregs(a800x600x16,sizeof(a800x600x16)/sizeof(a800x600x16[0]));
			break;
		case IGS_800x600x24:
			igs_setregs(a800x600x24,sizeof(a800x600x24)/sizeof(a800x600x24[0]));
			break;
		case IGS_1024x768x8:
			igs_setregs(a1024x768x8,sizeof(a1024x768x8)/sizeof(a1024x768x8[0]));
			break;
		case IGS_1024x768x16:
			igs_setregs(a1024x768x16,sizeof(a1024x768x16)/sizeof(a1024x768x16[0]));
			break;
		case IGS_1280x1024x8:
			igs_setregs(a1280x1024x8,sizeof(a1280x1024x8)/sizeof(a1280x1024x8[0]));
			break;
//		case IGS_1600x1200x8:
//			igs_setregs(a1600x1200x8,sizeof(a1600x1200x8)/sizeof(a1600x1200x8[0]));
//			break;
		default:
			pVidPort[0x3c0] = 0x20;	//video on
			return -EINVAL;
		}
	}
	// we probably want the video turned on
	pVidPort[0x3c0] = 0x20;

	mode = new_mode;
	return old_mode;
}

