/*
 *		IPPORTFW_LBM IP-Port Forwarding Virtual Server load balancing module
 *
 *
 * Version:	@(#)ip_portfw_lbm.c  0.01      99/01/05
 *
 * Author:	Matthew Kellett <matthewk@corelcomputer.com>
 *
 *  Converted from ip_masq_app.c
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version
 *	2 of the License, or (at your option) any later version.
 *
 * Fixes:
 *
 *
 * FIXME:
 *	
 */

#include <linux/config.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/skbuff.h>
#include <linux/in.h>
#include <linux/ip.h>
#include <net/protocol.h>
#include <net/tcp.h>
#include <net/udp.h>
#include <asm/system.h>
#include <linux/stat.h>
#include <linux/proc_fs.h>
#include <net/ip_masq.h>

static const char *strProt[] = {"UDP","TCP"};

static __inline__ const char * masq_proto_name(unsigned proto)
{
        return strProt[proto==IPPROTO_TCP];
}


static struct symbol_table ip_portfw_lbm_syms = {
#include <linux/symtab_begin.h>
	X(register_ip_portfw_lbm),
	X(unregister_ip_portfw_lbm),
#include <linux/symtab_end.h>
};

/*
 * 	will hold ip_portfw lbm. list head
 */

struct ip_portfw_lbm *ip_portfw_lbm_base=NULL;

/*
 * 	ip_portfw_lbm registration routine
 *	port: host byte order.
 */

int register_ip_portfw_lbm(struct ip_portfw_lbm *lbm)
{
	unsigned long flags;
	if (!lbm) {
		printk("register_ip_portfw_lbm(): NULL arg\n");
		return -EINVAL;
	}
	printk(KERN_DEBUG "register_ip_portfw_lbm: Registering %p\n", lbm);
	lbm->n_attach = 0;
	save_flags(flags);
	cli();
	lbm->next = ip_portfw_lbm_base;
	ip_portfw_lbm_base = lbm; 
	restore_flags(flags);
	
	return 0;
}

/*
 * 	ip_portfw_lbm unreg. routine.
 */

int unregister_ip_portfw_lbm(struct ip_portfw_lbm *lbm)
{
	struct ip_portfw_lbm **lbm_p;
	unsigned long flags;
	if (!lbm) {
		printk("unregister_ip_portfw_lbm(): NULL arg\n");
		return -EINVAL;
	}
	/*
	 * only allow unregistration if it has no attachments
	 */
	if (lbm->n_attach)  {
		printk("unregister_ip_portfw_lbm(): has %d attachments. failed\n",
			   lbm->n_attach);
		return -EINVAL;
	}

	printk(KERN_DEBUG "unregister_ip_portfw_lbm: Unregistering %p\n", lbm);

	save_flags(flags);
	cli();
	for (lbm_p = &ip_portfw_lbm_base; *lbm_p ; lbm_p = &(*lbm_p)->next)
		if (lbm == (*lbm_p))  {
			*lbm_p = lbm->next;
			restore_flags(flags);
			return 0;
		}
	
	restore_flags(flags);
	return -EINVAL;
}

/*
 *	get ip_portfw_lbm object by its proto and port (net byte order).
 */

struct ip_portfw_lbm * ip_portfw_lbm_get(char* name)
{
	struct ip_portfw_lbm *lbm;

	for(lbm = ip_portfw_lbm_base; lbm ; lbm = lbm->next) {
		if (strncmp(lbm->name, name, IPPORTFW_LBM_NAME_LEN) == 0)
			 return lbm;
	}
	return NULL;
}

/*
 *	ip_portfw_lbm object binding related funcs.
 */

/*
 * 	change ip_portfw object's number of bindings
 */

static __inline__ int ip_portfw_lbm_bind_chg(struct ip_portfw_lbm *lbm, int delta)
{
	unsigned long flags;
	int n_at;
	if (!lbm) return -1;
	save_flags(flags);
	cli();
	n_at = lbm->n_attach + delta;
	if (n_at < 0) {
		restore_flags(flags);
		printk("ip_portfw_lbm: tried to set n_attach < 0 for (%s) ip_portfw_lbm object.\n",
			lbm->name);
		return -1;
	}
	printk(KERN_DEBUG "ip_portfw_lbm_bind_chg: Adding %d attachments to %p\n", delta, lbm);
	lbm->n_attach = n_at;
	restore_flags(flags);
	return 0;
}

/*
 *	Bind ip_masq to its ip_portfw_lbm based on proto and dport ALREADY
 *	set in ip_masq struct. Also calls constructor.
 */

struct ip_portfw_lbm * ip_portfw_bind_lbm(struct ip_portfw *ippfvs, char *lbm_name)
{
	struct ip_portfw_lbm * lbm;
	lbm = ip_portfw_lbm_get(lbm_name);
	if (lbm != NULL) {
		/*
		 *	don't allow binding if already bound
		 */
		
		if (ippfvs->lbm != NULL) {
			printk("ip_portfw_bind_lbm() called for already bound object.\n");
			return ippfvs->lbm;
		}
		printk(KERN_DEBUG "ip_portfw_bind_lbm: Attach %s(%p) to %p\n", lbm_name, lbm, ippfvs);
		ippfvs->lbm = lbm;
		if (lbm->lbm_init_1) lbm->lbm_init_1(&(ippfvs->lbm_data));
		ip_portfw_lbm_bind_chg(lbm, +1);
	}
	return lbm;
}

/*
 * 	Unbind ms from type object and call ms destructor (does not kfree()).
 */

int ip_portfw_unbind_lbm(struct ip_portfw *ippfvs)
{
	struct ip_portfw_lbm * lbm;
	lbm = ippfvs->lbm;
	if (lbm != NULL) {
		if (lbm->lbm_done_1) lbm->lbm_done_1(&(ippfvs->lbm_data));
		ippfvs->lbm = NULL;
		ip_portfw_lbm_bind_chg(lbm, -1);
		printk(KERN_DEBUG "ip_portfw_unbind_lbm: Detach %p from %p\n", lbm, ippfvs);
	}
	return (lbm != NULL);
}


/*
 *	Server choice hook. Will call bound ip_portfw_lbm specific function
 *	called by ip_fw_masquerade()
 */

struct server *ip_portfw_lbm_lookup(struct ip_portfw *ippfvs)
{
	struct ip_portfw_lbm *lbm;
	struct server *chosen;
	/*
	 *	check if application masquerading is bound to
	 *	this ip_portfw.
	 */
	
	lbm = ippfvs->lbm;
	if ( lbm != NULL) {
		/*
		 *	Call private output hook function 
		 */
		if ( lbm->lookup != NULL ) {
			chosen = lbm->lookup(ippfvs->srv_lst, &(ippfvs->lbm_data));
			printk(KERN_DEBUG "ip_portfw_lbm_lookup: Module %p(ext srv:%p)->lookup(%p, %p) chose server %p\n", lbm, ippfvs, ippfvs->srv_lst, ippfvs->lbm_data, chosen);
			return chosen;
		}
			
	}
	/*
	 * Return next server round-robin
	 */
	ippfvs->next_srv = ippfvs->next_srv->next;
	if (!ippfvs->next_srv)
		ippfvs->next_srv = ippfvs->srv_lst; 

	printk(KERN_DEBUG "ip_portfw_lbm_lookup: Default chose server %p\n", ippfvs->next_srv);
	
	return ippfvs->next_srv;
}

/*
 *	Server change hook. Will call bound ip_portfw_lbm specific function
 *	called by ip_portfw_add() or ip_portfw_del()
 */

int ip_portfw_lbm_server_change(struct ip_portfw *ippfvs)
{
	
	struct ip_portfw_lbm *lbm;
	/*
	 *	check if application masquerading is bound to
	 *	this ip_portfw.
	 */
	
	lbm = ippfvs->lbm;
	if ( lbm != NULL) {
		/*
		 *	Call private output hook function 
		 */
		if ( lbm->server_change != NULL )
		{
			printk(KERN_DEBUG "ip_portfw_lbm_server_change: Module %p->server_change(%p, %p)\n", lbm, ippfvs->srv_lst, ippfvs->lbm_data);
			return lbm->server_change(ippfvs->srv_lst, &(ippfvs->lbm_data));
		}
	}
	/*
	 * Handle next pointer for default
	 */
	ippfvs->next_srv = ippfvs->srv_lst;
	printk(KERN_DEBUG "ip_portfw_server_change: Default next_srv(%p)\n", ippfvs->next_srv);
	return 1;
}


/*
 *	/proc/ip_portfw_lbm entry function
 */

int ip_portfw_lbm_getinfo(char *buffer, char **start, off_t offset, int length, int dummy)
{
	off_t pos=0, begin=0;
	int len=0;
	struct ip_portfw_lbm * lbm;

	if (offset < 28)
		len=sprintf(buffer,"%-27s\n", "n_attach name");
	pos = 28;

	for (lbm = ip_portfw_lbm_base; lbm ; lbm = lbm->next) {
		/* 
		 * If you change the length of this sprintf, then all
		 * the length calculations need fixing too!
		 * Line length = 26 (7 + 2 + 17)
		 */
		pos += 28;
		if (pos < offset)
			continue;

		len += sprintf(buffer+len, "%-7d  %-17s\n",
					   lbm->n_attach,
					   lbm->name);

		if(len >= length)
			goto done;
	}
done:
	begin = len - (pos - offset);
	*start = buffer + begin;
	len -= begin;
	if (len > length)
		len = length;	
	return len;
}


/*
 *	Initialization routine
 */

int ip_portfw_lbm_init(void)
{
        
	register_symtab (&ip_portfw_lbm_syms);
#ifdef CONFIG_PROC_FS        
	proc_net_register(&(struct proc_dir_entry) {
		PROC_NET_IPPFVS_LBM, 13, "ip_portfw_lbm",
		S_IFREG | S_IRUGO, 1, 0, 0,
		0, &proc_net_inode_operations,
		ip_portfw_lbm_getinfo
	});
#endif        
	return 0;
}
